@@ -19,15 +19,20 @@ Naming Conventions
 """
 
 from __future__ import division
-import os
-import sys
-import warnings
+import os, sys, warnings, gzip
+
+import numpy as npy
+
 def _fn_name(): return sys._getframe(1).f_code.co_name
 
-import cairo
+try:
+   import cairo
+except ImportError:
+   raise ImportError("Cairo backend requires that pycairo is installed.")
+
 _version_required = (1,2,0)
 if cairo.version_info < _version_required:
-   raise SystemExit ("Pycairo %d.%d.%d is installed\n"
+   raise ImportError ("Pycairo %d.%d.%d is installed\n"
                      "Pycairo %d.%d.%d or later is required"
                      % (cairo.version_info + _version_required))
 backend_version = cairo.version
@@ -35,25 +40,22 @@ del _version_required
 
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
      FigureManagerBase, FigureCanvasBase
-from matplotlib.cbook      import enumerate, izip
-from matplotlib.figure     import Figure
-from matplotlib.mathtext   import math_parse_s_ft2font
-import matplotlib.numerix as numx
-from matplotlib.transforms import Bbox
-
+from matplotlib.cbook        import is_string_like
+from matplotlib.figure       import Figure
+from matplotlib.mathtext     import MathTextParser
+from matplotlib.path         import Path
+from matplotlib.transforms   import Bbox, Affine2D
+from matplotlib.font_manager import ttfFontProperty
+from matplotlib import rcParams
 
 _debug = False
 #_debug = True
 
-# Image formats that this backend supports - for print_figure()
-IMAGE_FORMAT          = ['eps', 'pdf', 'png', 'ps', 'svg']
-IMAGE_FORMAT_DEFAULT  = 'png'
-
 # Image::color_conv(format) for draw_image()
 if sys.byteorder == 'little':
-   BYTE_FORMAT = 0 # BGRA
+    BYTE_FORMAT = 0 # BGRA
 else:
-   BYTE_FORMAT = 1 # ARGB
+    BYTE_FORMAT = 1 # ARGB
 
 
 class RendererCairo(RendererBase):
@@ -91,11 +93,11 @@ class RendererCairo(RendererBase):
         self.dpi = dpi
         self.text_ctx = cairo.Context (
            cairo.ImageSurface (cairo.FORMAT_ARGB32,1,1))
-
+        self.mathtext_parser = MathTextParser('Cairo')
 
     def set_ctx_from_surface (self, surface):
-       self.ctx = cairo.Context (surface)
-       self.ctx.save() # restore, save  - when call new_gc()
+        self.ctx = cairo.Context (surface)
+        self.ctx.save() # restore, save  - when call new_gc()
 
 
     def set_width_height(self, width, height):
@@ -107,47 +109,51 @@ class RendererCairo(RendererBase):
         # font transform?
 
 
-    def _fill_and_stroke (self, ctx, fill_c):
-        #assert fill_c or stroke_c
-
-        #_.ctx.save()
-
-        if fill_c:
+    def _fill_and_stroke (self, ctx, fill_c, alpha):
+        if fill_c is not None:
             ctx.save()
-            ctx.set_source_rgb (*fill_c)
-            #if stroke_c:   # always (implicitly) set at the moment
+            if len(fill_c) == 3:
+                ctx.set_source_rgba (fill_c[0], fill_c[1], fill_c[2], alpha)
+            else:
+                ctx.set_source_rgba (fill_c[0], fill_c[1], fill_c[2], alpha*fill_c[3])
             ctx.fill_preserve()
-            #else:
-            #    ctx.fill()
             ctx.restore()
-
-        #if stroke_c:                      # always stroke
-            #ctx.set_source_rgb (stroke_c) # is already set
         ctx.stroke()
 
-        #_.ctx.restore() # revert to the default attributes
+
+    #@staticmethod
+    def convert_path(ctx, tpath):
+        for points, code in tpath.iter_segments():
+            if code == Path.MOVETO:
+                ctx.move_to(*points)
+            elif code == Path.LINETO:
+                ctx.line_to(*points)
+            elif code == Path.CURVE3:
+                ctx.curve_to(points[0], points[1],
+                             points[0], points[1],
+                             points[2], points[3])
+            elif code == Path.CURVE4:
+                ctx.curve_to(*points)
+            elif code == Path.CLOSEPOLY:
+                ctx.close_path()
+    convert_path = staticmethod(convert_path)
 
 
-    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2,
-                 rotation):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        # this implementation draws circular arcs where width=height
-        # FIXME
-        # to get a proper arc of width/height you can use translate() and
-        # scale(), see cairo_arc() manual page
+    def draw_path(self, gc, path, transform, rgbFace=None):
+        if len(path.vertices) > 18980:
+           raise ValueError("The Cairo backend can not draw paths longer than 18980 points.")
 
         ctx = gc.ctx
-        ctx.save()
-        ctx.new_path()  # prevent cairo_arc() adding to current path
-        ctx.rotate(rotation)
-        radius = (height + width) / 4
-        ctx.arc (x, self.height - y, radius,
-                 angle1 * numx.pi/180.0, angle2 * numx.pi/180.0)
-        ctx.restore()
-        self._fill_and_stroke (ctx, rgbFace)
+        transform = transform + \
+            Affine2D().scale(1.0, -1.0).translate(0, self.height)
+        tpath = transform.transform_path(path)
 
+        ctx.new_path()
+        self.convert_path(ctx, tpath)
 
-    def draw_image(self, x, y, im, bbox):
+        self._fill_and_stroke(ctx, rgbFace, gc.get_alpha())
+
+    def draw_image(self, x, y, im, bbox, clippath=None, clippath_trans=None):
         # bbox - not currently used
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
 
@@ -155,8 +161,7 @@ class RendererCairo(RendererBase):
 
         rows, cols, buf = im.color_conv (BYTE_FORMAT)
         surface = cairo.ImageSurface.create_for_data (
-                      buf, cairo.FORMAT_ARGB32, rows, cols, rows*4)
-
+                      buf, cairo.FORMAT_ARGB32, cols, rows, cols*4)
         # function does not pass a 'gc' so use renderer.ctx
         ctx = self.ctx
         y = self.height - y - rows
@@ -165,118 +170,6 @@ class RendererCairo(RendererBase):
 
         im.flipud_out()
 
-
-    def draw_line(self, gc, x1, y1, x2, y2):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        ctx = gc.ctx
-        ctx.new_path()
-        ctx.move_to (x1, self.height - y1)
-        ctx.line_to (x2, self.height - y2)
-        self._fill_and_stroke (ctx, None)
-
-
-    def draw_lines(self, gc, x, y, transform=None):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-
-        if transform:
-            if transform.need_nonlinear():
-                x, y = transform.nonlinear_only_numerix(x, y)
-            x, y = transform.numerix_x_y(x, y)
-
-        ctx = gc.ctx
-        matrix_old = ctx.get_matrix()
-        ctx.set_matrix (self.matrix_flipy)
-
-        points = izip(x,y)
-        x, y = points.next()
-        ctx.new_path()
-        ctx.move_to (x, y)
-
-        for x,y in points:
-            ctx.line_to (x, y)
-        self._fill_and_stroke (ctx, None)
-
-        ctx.set_matrix (matrix_old)
-
-
-    def draw_markers_OLD(self, gc, path, rgbFace, x, y, transform):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-
-        ctx = gc.ctx
-
-        if transform.need_nonlinear():
-            x,y = transform.nonlinear_only_numerix(x, y)
-
-        x, y = transform.numerix_x_y(x, y) # do nonlinear and affine transform
-
-        # TODO - use cairo transform
-        # matrix worked for dotted lines, but not markers in line_styles.py
-        # it upsets/transforms generate_path() ?
-        # need to flip y too, and update generate_path() ?
-        # the a,b,c,d,tx,ty affine which transforms x and y
-        #vec6 = transform.as_vec6_val() # not used (yet)
-        #matrix_old = ctx.get_matrix()
-        #ctx.set_matrix (cairo.Matrix (*vec6))
-
-        path_list = [path.vertex() for i in range(path.total_vertices())]
-
-        def generate_path (path_list):
-           for code, xp, yp in path_list:
-               if code == agg.path_cmd_move_to:
-                  ctx.move_to (xp, -yp)
-               elif code == agg.path_cmd_line_to:
-                  ctx.line_to (xp, -yp)
-               elif code == agg.path_cmd_end_poly:
-                  ctx.close_path()
-
-        for x,y in izip(x,y):
-            ctx.save()
-            ctx.new_path()
-            ctx.translate(x, self.height - y)
-            generate_path (path_list)
-
-            self._fill_and_stroke (ctx, rgbFace)
-
-            ctx.restore() # undo translate()
-
-        #ctx.set_matrix(matrix_old)
-
-
-    def draw_point(self, gc, x, y):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        # render by drawing a 0.5 radius circle
-        ctx = gc.ctx
-        ctx.new_path()
-        ctx.arc (x, self.height - y, 0.5, 0, 2*numx.pi)
-        self._fill_and_stroke (ctx, gc.get_rgb())
-
-
-    def draw_polygon(self, gc, rgbFace, points):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-
-        ctx = gc.ctx
-        matrix_old = ctx.get_matrix()
-        ctx.set_matrix (self.matrix_flipy)
-
-        ctx.new_path()
-        x, y = points[0]
-        ctx.move_to (x, y)
-        for x,y in points[1:]:
-            ctx.line_to (x, y)
-        ctx.close_path()
-
-        self._fill_and_stroke (ctx, rgbFace)
-
-        ctx.set_matrix (matrix_old)
-
-    def draw_rectangle(self, gc, rgbFace, x, y, width, height):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        ctx = gc.ctx
-        ctx.new_path()
-        ctx.rectangle (x, self.height - y - height, width, height)
-        self._fill_and_stroke (ctx, rgbFace)
-
-
     def draw_text(self, gc, x, y, s, prop, angle, ismath=False):
         # Note: x,y are device/display coords, not user-coords, unlike other
         # draw_* methods
@@ -293,72 +186,49 @@ class RendererCairo(RendererBase):
                                  self.fontangles [prop.get_style()],
                                  self.fontweights[prop.get_weight()])
 
-           # size = prop.get_size_in_points() * self.dpi.get() / 96.0
-           size = prop.get_size_in_points() * self.dpi.get() / 72.0
+           size = prop.get_size_in_points() * self.dpi / 72.0
 
            ctx.save()
            if angle:
-              ctx.rotate (-angle * numx.pi / 180)
+              ctx.rotate (-angle * npy.pi / 180)
            ctx.set_font_size (size)
-           ctx.show_text (s)
+           ctx.show_text (s.encode("utf-8"))
            ctx.restore()
 
-
     def _draw_mathtext(self, gc, x, y, s, prop, angle):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        # mathtext using the gtk/gdk method
 
-        #if numx.which[0] == "numarray":
-        #   warnings.warn("_draw_mathtext() currently works for numpy, but "
-        #                 "not numarray")
-        #   return
+        ctx = gc.ctx
+        width, height, descent, glyphs, rects = self.mathtext_parser.parse(
+            s, self.dpi, prop)
 
-        #if not HAVE_CAIRO_NUMPY:
-        #    warnings.warn("cairo with Numeric support is required for "
-        #                  "_draw_mathtext()")
-        #    return
+        ctx.save()
+        ctx.translate(x, y)
+        if angle:
+           ctx.rotate (-angle * npy.pi / 180)
 
-        size = prop.get_size_in_points()
-        width, height, fonts = math_parse_s_ft2font(
-            s, self.dpi.get(), size)
+        for font, fontsize, s, ox, oy in glyphs:
+           ctx.new_path()
+           ctx.move_to(ox, oy)
 
-        if angle==90:
-            width, height = height, width
-            x -= width
-        y -= height
+           fontProp = ttfFontProperty(font)
+           ctx.save()
+           ctx.select_font_face (fontProp.name,
+                                 self.fontangles [fontProp.style],
+                                 self.fontweights[fontProp.weight])
 
-        imw, imh, s = fonts[0].image_as_str()
-        N = imw*imh
+           size = fontsize * self.dpi / 72.0
+           ctx.set_font_size(size)
+           ctx.show_text(s.encode("utf-8"))
+           ctx.restore()
 
-        # a numpixels by num fonts array
-        Xall = numx.zeros((N,len(fonts)), typecode=numx.UInt8)
+        for ox, oy, w, h in rects:
+           ctx.new_path()
+           ctx.rectangle (ox, oy, w, h)
+           ctx.set_source_rgb (0, 0, 0)
+           ctx.fill_preserve()
 
-        for i, font in enumerate(fonts):
-            if angle == 90:
-                font.horiz_image_to_vert_image() # <-- Rotate
-            imw, imh, s = font.image_as_str()
-            Xall[:,i] = numx.fromstring(s, numx.UInt8)
-
-        # get the max alpha at each pixel
-        Xs = numx.mlab.max (Xall,1)
-
-        # convert it to it's proper shape
-        Xs.shape = imh, imw
-
-        pa = numx.zeros(shape=(imh,imw,4), typecode=numx.UInt8)
-        rgb = gc.get_rgb()
-        pa[:,:,0] = int(rgb[0]*255)
-        pa[:,:,1] = int(rgb[1]*255)
-        pa[:,:,2] = int(rgb[2]*255)
-        pa[:,:,3] = Xs
-
-        ## works for numpy pa, not a numarray pa
-        #surface = cairo.ImageSurface.create_for_array (pa)
-        surface = cairo.ImageSurface.create_for_data (pa, cairo.FORMAT_ARGB32,
-                                                      imw, imh, imw*4)
-        gc.ctx.set_source_surface (surface, x, y)
-        gc.ctx.paint()
-        #gc.ctx.show_surface (surface, imw, imh)
+        ctx.restore()
 
 
     def flipy(self):
@@ -373,12 +243,12 @@ class RendererCairo(RendererBase):
         return self.width, self.height
 
 
-    def get_text_width_height(self, s, prop, ismath):
+    def get_text_width_height_descent(self, s, prop, ismath):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
         if ismath:
-            width, height, fonts = math_parse_s_ft2font(
-               s, self.dpi.get(), prop.get_size_in_points())
-            return width, height
+            width, height, descent, fonts, used_characters = self.mathtext_parser.parse(
+               s, self.dpi, prop)
+            return width, height, descent
 
         ctx = self.text_ctx
         ctx.save()
@@ -389,18 +259,17 @@ class RendererCairo(RendererBase):
         # Cairo (says it) uses 1/96 inch user space units, ref: cairo_gstate.c
         # but if /96.0 is used the font is too small
 
-        #size = prop.get_size_in_points() * self.dpi.get() / 96.0
-        size = prop.get_size_in_points() * self.dpi.get() / 72.0
+        size = prop.get_size_in_points() * self.dpi / 72.0
 
         # problem - scale remembers last setting and font can become
         # enormous causing program to crash
         # save/restore prevents the problem
         ctx.set_font_size (size)
 
-        w, h = ctx.text_extents (s)[2:4]
+        y_bearing, w, h = ctx.text_extents (s)[1:4]
         ctx.restore()
 
-        return w, h
+        return w, h, h + y_bearing
 
 
     def new_gc(self):
@@ -412,7 +281,7 @@ class RendererCairo(RendererBase):
 
     def points_to_pixels(self, points):
         if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        return points/72.0 * self.dpi.get()
+        return points/72.0 * self.dpi
 
 
 class GraphicsContextCairo(GraphicsContextBase):
@@ -455,8 +324,10 @@ class GraphicsContextCairo(GraphicsContextBase):
 
     def set_clip_rectangle(self, rectangle):
         self._cliprect = rectangle
+        if rectangle is None:
+           return
 
-        x,y,w,h = rectangle
+        x,y,w,h = rectangle.bounds
         # pixel-aligned clip-regions are faster
         x,y,w,h = round(x), round(y), round(w), round(h)
         ctx = self.ctx
@@ -467,23 +338,39 @@ class GraphicsContextCairo(GraphicsContextBase):
         # in fill_and_stroke() inside ctx.save() ... ctx.restore()
 
 
+    def set_clip_path(self, path):
+        if path is not None:
+            tpath, affine = path.get_transformed_path_and_affine()
+            ctx = self.ctx
+            ctx.new_path()
+            affine = affine + Affine2D().scale(1.0, -1.0).translate(0.0, self.renderer.height)
+            tpath = affine.transform_path(tpath)
+            RendererCairo.convert_path(ctx, tpath)
+            ctx.clip()
+
+
     def set_dashes(self, offset, dashes):
         self._dashes = offset, dashes
         if dashes == None:
             self.ctx.set_dash([], 0)  # switch dashes off
         else:
             self.ctx.set_dash (
-               self.renderer.points_to_pixels (numx.asarray(dashes)), offset)
+               self.renderer.points_to_pixels (npy.asarray(dashes)), offset)
 
 
     def set_foreground(self, fg, isRGB=None):
         GraphicsContextBase.set_foreground(self, fg, isRGB)
-        self.ctx.set_source_rgb(*self._rgb)
-
+        if len(self._rgb) == 3:
+            self.ctx.set_source_rgb(*self._rgb)
+        else:
+            self.ctx.set_source_rgba(*self._rgb)
 
     def set_graylevel(self, frac):
         GraphicsContextBase.set_graylevel(self, frac)
-        self.ctx.set_source_rgb(*self._rgb)
+        if len(self._rgb) == 3:
+            self.ctx.set_source_rgb(*self._rgb)
+        else:
+            self.ctx.set_source_rgba(*self._rgb)
 
 
     def set_joinstyle(self, js):
@@ -512,37 +399,7 @@ def new_figure_manager(num, *args, **kwargs): # called by backends/__init__.py
 
 
 class FigureCanvasCairo (FigureCanvasBase):
-    def print_figure(self, fo, dpi=150, facecolor='w', edgecolor='w',
-                     orientation='portrait', format=None, **kwargs):
-        if _debug: print '%s.%s()' % (self.__class__.__name__, _fn_name())
-        # settings for printing
-        self.figure.dpi.set(dpi)
-        self.figure.set_facecolor(facecolor)
-        self.figure.set_edgecolor(edgecolor)
-
-        if format is None and isinstance (fo, basestring):
-            # get format from filename extension
-            format = os.path.splitext(fo)[1][1:]
-
-        if format is not None:
-            format = format.lower()
-
-        if format == 'png':
-            self._save_png (fo)
-        elif format in ('pdf', 'ps', 'svg'):
-            self._save (fo, format, orientation, **kwargs)
-        elif format == 'eps': # backend_ps for eps
-            from backend_ps import FigureCanvasPS  as FigureCanvas
-            fc = FigureCanvas(self.figure)
-            fc.print_figure (filename, dpi, facecolor, edgecolor,
-                             orientation, **kwargs)
-        else:
-            warnings.warn('Format "%s" is not supported.\n'
-                          'Supported formats: '
-                          '%s.' % (format, ', '.join(IMAGE_FORMAT)))
-
-
-    def _save_png (self, fobj):
+    def print_png(self, fobj, *args, **kwargs):
         width, height = self.get_width_height()
 
         renderer = RendererCairo (self.figure.dpi)
@@ -553,13 +410,27 @@ class FigureCanvasCairo (FigureCanvasBase):
         self.figure.draw (renderer)
         surface.write_to_png (fobj)
 
+    def print_pdf(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'pdf', *args, **kwargs)
 
-    def _save (self, fo, format, orientation, **kwargs):
+    def print_ps(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'ps', *args, **kwargs)
+
+    def print_svg(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'svg', *args, **kwargs)
+
+    def print_svgz(self, fobj, *args, **kwargs):
+        return self._save(fobj, 'svgz', *args, **kwargs)
+
+    def get_default_filetype(self):
+        return rcParams['cairo.format']
+
+    def _save (self, fo, format, **kwargs):
         # save PDF/PS/SVG
         orientation = kwargs.get('orientation', 'portrait')
 
         dpi = 72
-        self.figure.dpi.set (dpi)
+        self.figure.dpi = dpi
         w_in, h_in = self.figure.get_size_inches()
         width_in_points, height_in_points = w_in * dpi, h_in * dpi
 
@@ -577,10 +448,15 @@ class FigureCanvasCairo (FigureCanvasBase):
                 raise RuntimeError ('cairo has not been compiled with PDF '
                                     'support enabled')
             surface = cairo.PDFSurface (fo, width_in_points, height_in_points)
-        elif format == 'svg':
+        elif format in ('svg', 'svgz'):
             if not cairo.HAS_SVG_SURFACE:
                 raise RuntimeError ('cairo has not been compiled with SVG '
                                     'support enabled')
+            if format == 'svgz':
+                filename = fo
+                if is_string_like(fo):
+                    fo = open(fo, 'wb')
+                fo = gzip.GzipFile(None, 'wb', fileobj=fo)
             surface = cairo.SVGSurface (fo, width_in_points, height_in_points)
         else:
            warnings.warn ("unknown format: %s" % format)
@@ -593,7 +469,7 @@ class FigureCanvasCairo (FigureCanvasBase):
         ctx = renderer.ctx
 
         if orientation == 'landscape':
-            ctx.rotate (numx.pi/2)
+            ctx.rotate (npy.pi/2)
             ctx.translate (0, -height_in_points)
             # cairo/src/cairo_ps_surface.c
             # '%%Orientation: Portrait' is always written to the file header
