@@ -3,12 +3,16 @@ from __future__ import division
 import os, sys
 def fn_name(): return sys._getframe(1).f_code.co_name
 
-import gobject
-import gtk; gdk = gtk.gdk
-import pango
+try:
+    import gobject
+    import gtk; gdk = gtk.gdk
+    import pango
+except ImportError:
+    raise ImportError("Gtk* backend requires pygtk to be installed.")
+
 pygtk_version_required = (2,2,0)
 if gtk.pygtk_version < pygtk_version_required:
-    raise SystemExit ("PyGTK %d.%d.%d is installed\n"
+    raise ImportError ("PyGTK %d.%d.%d is installed\n"
                       "PyGTK %d.%d.%d or later is required"
                       % (gtk.pygtk_version + pygtk_version_required))
 del pygtk_version_required
@@ -19,14 +23,13 @@ from matplotlib._pylab_helpers import Gcf
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
      FigureManagerBase, FigureCanvasBase, NavigationToolbar2, cursors
 from matplotlib.backends.backend_gdk import RendererGDK, FigureCanvasGDK
-from matplotlib.cbook import is_string_like, enumerate
+from matplotlib.cbook import is_string_like, is_writable_file_like
 from matplotlib.colors import colorConverter
 from matplotlib.figure import Figure
-import matplotlib.numerix as numerix
-from matplotlib.numerix import asarray, fromstring, UInt8, zeros, \
-     where, transpose, nonzero, indices, ones, nx
 from matplotlib.widgets import SubplotTool
 
+from matplotlib import lines
+from matplotlib import cbook
 
 backend_version = "%d.%d.%d" % gtk.pygtk_version
 
@@ -37,14 +40,6 @@ _debug = False
 # see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
 PIXELS_PER_INCH = 96
 
-# Image formats that this backend supports - for FileChooser and print_figure()
-IMAGE_FORMAT = ['bmp', 'eps', 'jpg', 'png', 'ps', 'svg']
-# pdf not ready yet
-#IMAGE_FORMAT  = ['bmp', 'eps', 'jpg', 'png', 'pdf', 'ps', 'svg']
-IMAGE_FORMAT.sort()
-IMAGE_FORMAT_DEFAULT  = 'png'
-
-
 cursord = {
     cursors.MOVE          : gdk.Cursor(gdk.FLEUR),
     cursors.HAND          : gdk.Cursor(gdk.HAND2),
@@ -67,6 +62,7 @@ def draw_if_interactive():
         if figManager is not None:
             figManager.canvas.draw()
 
+
 def show(mainloop=True):
     """
     Show all the figures and enter the gtk main loop
@@ -75,14 +71,15 @@ def show(mainloop=True):
     for manager in Gcf.get_all_fig_managers():
         manager.window.show()
 
-    if mainloop and gtk.main_level() == 0:
+    if mainloop and gtk.main_level() == 0 and \
+                            len(Gcf.get_all_fig_managers())>0:
         gtk.main()
 
 def new_figure_manager(num, *args, **kwargs):
     """
     Create a new figure manager instance
     """
-    FigureClass = kwargs.pop('FigureClass', Figure)    
+    FigureClass = kwargs.pop('FigureClass', Figure)
     thisFig = FigureClass(*args, **kwargs)
     canvas = FigureCanvasGTK(thisFig)
     manager = FigureManagerGTK(canvas, num)
@@ -102,19 +99,71 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
                65362 : 'up',
                65363 : 'right',
                65364 : 'down',
+               65307 : 'escape',
+               65470 : 'f1',
+               65471 : 'f2',
+               65472 : 'f3',
+               65473 : 'f4',
+               65474 : 'f5',
+               65475 : 'f6',
+               65476 : 'f7',
+               65477 : 'f8',
+               65478 : 'f9',
+               65479 : 'f10',
+               65480 : 'f11',
+               65481 : 'f12',
+               65300 : 'scroll_lock',
+               65299 : 'break',
+               65288 : 'backspace',
+               65293 : 'enter',
+               65379 : 'insert',
+               65535 : 'delete',
+               65360 : 'home',
+               65367 : 'end',
+               65365 : 'pageup',
+               65366 : 'pagedown',
+               65438 : '0',
+               65436 : '1',
+               65433 : '2',
+               65435 : '3',
+               65430 : '4',
+               65437 : '5',
+               65432 : '6',
+               65429 : '7',
+               65431 : '8',
+               65434 : '9',
+               65451 : '+',
+               65453 : '-',
+               65450 : '*',
+               65455 : '/',
+               65439 : 'dec',
+               65421 : 'enter',
                }
 
+    # Setting this as a static constant prevents
+    # this resulting expression from leaking
+    event_mask = (gdk.BUTTON_PRESS_MASK   |
+                  gdk.BUTTON_RELEASE_MASK |
+                  gdk.EXPOSURE_MASK       |
+                  gdk.KEY_PRESS_MASK      |
+                  gdk.KEY_RELEASE_MASK    |
+                  gdk.ENTER_NOTIFY_MASK   |
+                  gdk.LEAVE_NOTIFY_MASK   |
+                  gdk.POINTER_MOTION_MASK |
+                  gdk.POINTER_MOTION_HINT_MASK)
+
     def __init__(self, figure):
         if _debug: print 'FigureCanvasGTK.%s' % fn_name()
         FigureCanvasBase.__init__(self, figure)
         gtk.DrawingArea.__init__(self)
 
-        self._idleID        = 0
+        self._idle_draw_id  = 0
         self._need_redraw   = True
         self._pixmap_width  = -1
         self._pixmap_height = -1
         self._lastCursor    = None
 
+        self.connect('scroll_event',         self.scroll_event)
         self.connect('button_press_event',   self.button_press_event)
         self.connect('button_release_event', self.button_release_event)
         self.connect('configure_event',      self.configure_event)
@@ -122,21 +171,34 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         self.connect('key_press_event',      self.key_press_event)
         self.connect('key_release_event',    self.key_release_event)
         self.connect('motion_notify_event',  self.motion_notify_event)
+        self.connect('leave_notify_event',   self.leave_notify_event)
+        self.connect('enter_notify_event',   self.enter_notify_event)
 
-        self.set_events(
-            gdk.BUTTON_PRESS_MASK   |
-            gdk.BUTTON_RELEASE_MASK |
-            gdk.EXPOSURE_MASK       |
-            gdk.KEY_PRESS_MASK      |
-            gdk.KEY_RELEASE_MASK    |
-            gdk.LEAVE_NOTIFY_MASK   |
-            gdk.POINTER_MOTION_MASK |
-            gdk.POINTER_MOTION_HINT_MASK)
+        self.set_events(self.__class__.event_mask)
 
         self.set_double_buffered(False)
         self.set_flags(gtk.CAN_FOCUS)
         self._renderer_init()
 
+        self._idle_event_id = gobject.idle_add(self.idle_event)
+
+    def destroy(self):
+        #gtk.DrawingArea.destroy(self)
+        gobject.source_remove(self._idle_event_id)
+        if self._idle_draw_id != 0:
+            gobject.source_remove(self._idle_draw_id)
+
+    def scroll_event(self, widget, event):
+        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        x = event.x
+        # flipy so y=0 is bottom of canvas
+        y = self.allocation.height - event.y
+        if event.direction==gdk.SCROLL_UP:
+            step = 1
+        else:
+            step = -1
+        FigureCanvasBase.scroll_event(self, x, y, step)
+        return False  # finish event propagation?
 
     def button_press_event(self, widget, event):
         if _debug: print 'FigureCanvasGTK.%s' % fn_name()
@@ -180,8 +242,14 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         FigureCanvasBase.motion_notify_event(self, x, y)
         return False  # finish event propagation?
 
+    def leave_notify_event(self, widget, event):
+        FigureCanvasBase.leave_notify_event(self, event)
+
+    def enter_notify_event(self, widget, event):
+        FigureCanvasBase.enter_notify_event(self, event)
+
     def _get_key(self, event):
-        if self.keyvald.has_key(event.keyval):
+        if event.keyval in self.keyvald:
             key = self.keyvald[event.keyval]
         elif event.keyval <256:
             key = chr(event.keyval)
@@ -202,7 +270,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
             return # empty fig
 
         # resize the figure (in inches)
-        dpi = self.figure.dpi.get()
+        dpi = self.figure.dpi
         self.figure.set_size_inches (w/dpi, h/dpi)
         self._need_redraw = True
 
@@ -223,10 +291,10 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
     def draw_idle(self):
         def idle_draw(*args):
             self.draw()
-            self._idleID = 0
+            self._idle_draw_id = 0
             return False
-        if self._idleID == 0:
-            self._idleID = gobject.idle_add(idle_draw)
+        if self._idle_draw_id == 0:
+            self._idle_draw_id = gobject.idle_add(idle_draw)
 
 
     def _renderer_init(self):
@@ -289,105 +357,72 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
                                        self._pixmap, x, y, x, y, w, h)
         return False  # finish event propagation?
 
+    filetypes = FigureCanvasBase.filetypes.copy()
+    filetypes['jpg'] = 'JPEG'
+    filetypes['jpeg'] = 'JPEG'
+    filetypes['png'] = 'Portable Network Graphics'
 
-    def print_figure(self, filename, dpi=None, facecolor='w', edgecolor='w',
-                     orientation='portrait', **kwargs):
-        # TODO - use gdk/cairo/agg print_figure?
-        if dpi is None: dpi = matplotlib.rcParams['savefig.dpi']
-        root, ext = os.path.splitext(filename)
-        ext = ext[1:]
-        if ext == '':
-            ext      = IMAGE_FORMAT_DEFAULT
-            filename = filename + '.' + ext
+    def print_jpeg(self, filename, *args, **kwargs):
+        return self._print_image(filename, 'jpeg')
+    print_jpg = print_jpeg
 
-        # save figure settings
-        origDPI       = self.figure.dpi.get()
-        origfacecolor = self.figure.get_facecolor()
-        origedgecolor = self.figure.get_edgecolor()
-        origWIn, origHIn = self.figure.get_size_inches()
+    def print_png(self, filename, *args, **kwargs):
+        return self._print_image(filename, 'png')
 
+    def _print_image(self, filename, format):
         if self.flags() & gtk.REALIZED == 0:
             # for self.window(for pixmap) and has a side effect of altering
             # figure width,height (via configure-event?)
             gtk.DrawingArea.realize(self)
 
-        self.figure.dpi.set(dpi)
-        self.figure.set_facecolor(facecolor)
-        self.figure.set_edgecolor(edgecolor)
+        width, height = self.get_width_height()
+        pixmap = gdk.Pixmap (self.window, width, height)
+        self._renderer.set_pixmap (pixmap)
+        self._render_figure(pixmap, width, height)
 
-        ext = ext.lower()
-        if ext in ('jpg', 'png'):          # native printing
-            width, height = self.get_width_height()
-            pixmap = gdk.Pixmap (self.window, width, height)
-            self._renderer.set_pixmap (pixmap)
-            self._render_figure(pixmap, width, height)
-
-            # jpg colors don't match the display very well, png colors match
-            # better
-            pixbuf = gdk.Pixbuf(gdk.COLORSPACE_RGB, 0, 8, width, height)
-            pixbuf.get_from_drawable(pixmap, pixmap.get_colormap(),
+        # jpg colors don't match the display very well, png colors match
+        # better
+        pixbuf = gdk.Pixbuf(gdk.COLORSPACE_RGB, 0, 8, width, height)
+        pixbuf.get_from_drawable(pixmap, pixmap.get_colormap(),
                                      0, 0, 0, 0, width, height)
 
-            # pixbuf.save() recognises 'jpeg' not 'jpg'
-            if ext == 'jpg': ext = 'jpeg'
+        if is_string_like(filename):
             try:
-                pixbuf.save(filename, ext)
+                pixbuf.save(filename, format)
             except gobject.GError, exc:
                 error_msg_gtk('Save figure failure:\n%s' % (exc,), parent=self)
-
-        elif ext in ('eps', 'ps', 'svg',):
-            if ext == 'svg':
-                from backend_svg import FigureCanvasSVG as FigureCanvas
+        elif is_writable_file_like(filename):
+            if hasattr(pixbuf, 'save_to_callback'):
+                def save_callback(buf, data=None):
+                    data.write(buf)
+                try:
+                    pixbuf.save_to_callback(save_callback, format, user_data=filename)
+                except gobject.GError, exc:
+                    error_msg_gtk('Save figure failure:\n%s' % (exc,), parent=self)
             else:
-                from backend_ps  import FigureCanvasPS  as FigureCanvas
-
-            try:
-                fc = self.switch_backends(FigureCanvas)
-                fc.print_figure(filename, dpi, facecolor, edgecolor,
-                                orientation, **kwargs)
-            except IOError, exc:
-                error_msg_gtk("Save figure failure:\n%s: %s" %
-                          (exc.filename, exc.strerror), parent=self)
-            except Exception, exc:
-                error_msg_gtk("Save figure failure:\n%s" % exc, parent=self)
-
-        elif ext in ('bmp', 'raw', 'rgb',):
-            try:
-                from backend_agg import FigureCanvasAgg  as FigureCanvas
-            except:
-                error_msg_gtk('Save figure failure:\n'
-                          'Agg must be installed to save as bmp, raw and rgb',
-                          parent=self)
-            else:
-                fc = self.switch_backends(FigureCanvas)
-                fc.print_figure(filename, dpi, facecolor, edgecolor,
-                                orientation, **kwargs)
-
-        elif ext in ('pdf',):
-            try:
-                from backend_cairo import FigureCanvasCairo  as FigureCanvas
-            except:
-                error_msg_gtk('Save figure failure:\n'
-                          'Cairo must be installed to save as pdf',
-                          parent=self)
-            else:
-                fc = self.switch_backends(FigureCanvas)
-                fc.print_figure(filename, dpi, facecolor, edgecolor,
-                                orientation, **kwargs)
-
+                raise ValueError("Saving to a Python file-like object is only supported by PyGTK >= 2.8")
         else:
-            error_msg_gtk('Format "%s" is not supported.\nSupported formats are %s.' %
-                      (ext, ', '.join(IMAGE_FORMAT)),
-                      parent=self)
+            raise ValueError("filename must be a path or a file-like object")
 
-        # restore figure settings
-        self.figure.dpi.set(origDPI)
-        self.figure.set_facecolor(origfacecolor)
-        self.figure.set_edgecolor(origedgecolor)
-        self.figure.set_size_inches(origWIn, origHIn)
-        self.figure.set_canvas(self)
+    def get_default_filetype(self):
+        return 'png'
 
 
+    def flush_events(self):
+        gtk.gdk.threads_enter()
+        while gtk.events_pending():
+            gtk.main_iteration(True)
+        gtk.gdk.flush()
+        gtk.gdk.threads_leave()
+
+    def start_event_loop(self,timeout):
+        FigureCanvasBase.start_event_loop_default(self,timeout)
+    start_event_loop.__doc__=FigureCanvasBase.start_event_loop_default.__doc__
+
+    def stop_event_loop(self):
+        FigureCanvasBase.stop_event_loop_default(self)
+    stop_event_loop.__doc__=FigureCanvasBase.stop_event_loop_default.__doc__
+
 class FigureManagerGTK(FigureManagerBase):
     """
     Public attributes
@@ -410,13 +445,17 @@ class FigureManagerGTK(FigureManagerBase):
         self.vbox.show()
 
         self.canvas.show()
+
+        # attach a show method to the figure  for pylab ease of use
+        self.canvas.figure.show = lambda *args: self.window.show()
+
         self.vbox.pack_start(self.canvas, True, True)
 
         self.toolbar = self._get_toolbar(canvas)
 
         # calculate size for window
-        w = int (self.canvas.figure.bbox.width())
-        h = int (self.canvas.figure.bbox.height())
+        w = int (self.canvas.figure.bbox.width)
+        h = int (self.canvas.figure.bbox.height)
 
         if self.toolbar is not None:
             self.toolbar.show()
@@ -442,13 +481,20 @@ class FigureManagerGTK(FigureManagerBase):
 
     def destroy(self, *args):
         if _debug: print 'FigureManagerGTK.%s' % fn_name()
+        self.vbox.destroy()
         self.window.destroy()
+        self.canvas.destroy()
+        self.toolbar.destroy()
+        self.__dict__.clear()
 
         if Gcf.get_num_fig_managers()==0 and \
                not matplotlib.is_interactive() and \
                gtk.main_level() >= 1:
             gtk.main_quit()
 
+    def show(self):
+        # show the figure window
+        self.window.show()
 
     def full_screen_toggle (self):
         self._full_screen_flag = not self._full_screen_flag
@@ -470,6 +516,8 @@ class FigureManagerGTK(FigureManagerBase):
             toolbar = None
         return toolbar
 
+    def set_window_title(self, title):
+        self.window.set_title(title)
 
     def resize(self, width, height):
         'set the canvas size in pixels'
@@ -497,10 +545,10 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         self.win = window
         gtk.Toolbar.__init__(self)
         NavigationToolbar2.__init__(self, canvas)
-        self._idleId = 0
+        self._idle_draw_id = 0
 
     def set_message(self, s):
-        if self._idleId == 0:
+        if self._idle_draw_id == 0:
             self.message.set_label(s)
 
     def set_cursor(self, cursor):
@@ -522,7 +570,7 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
 
         gc = drawable.new_gc()
 
-        height = self.canvas.figure.bbox.height()
+        height = self.canvas.figure.bbox.height
         y1 = height - y1
         y0 = height - y0
 
@@ -537,21 +585,20 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
                 return
 
             ax = event.inaxes
-            l,b,w,h = [int(val) for val in ax.bbox.get_bounds()]
+            l,b,w,h = [int(val) for val in ax.bbox.bounds]
             b = int(height)-(b+h)
             axrect = l,b,w,h
             self._imageBack = axrect, drawable.get_image(*axrect)
             drawable.draw_rectangle(gc, False, *rect)
-            self._idleId = 0
+            self._idle_draw_id = 0
         else:
             def idle_draw(*args):
-
                 drawable.draw_image(gc, imageBack, 0, 0, *lastrect)
                 drawable.draw_rectangle(gc, False, *rect)
-                self._idleId = 0
+                self._idle_draw_id = 0
                 return False
-            if self._idleId == 0:
-                self._idleId = gobject.idle_add(idle_draw)
+            if self._idle_draw_id == 0:
+                self._idle_draw_id = gobject.idle_add(idle_draw)
 
 
     def _init_toolbar(self):
@@ -587,11 +634,6 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         self.append_widget(self.message, None, None)
         self.message.show()
 
-        self.fileselect = FileSelection(title='Save the figure',
-                                        parent=self.win,)
-
-
-
     def _init_toolbar2_4(self):
         basedir = os.path.join(matplotlib.rcParams['datapath'],'images')
         self.tooltips = gtk.Tooltips()
@@ -622,14 +664,24 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
 
         self.show_all()
 
-        self.fileselect = FileChooserDialog(title='Save the figure',
-                                            parent=self.win,)
-
+    def get_filechooser(self):
+        if gtk.pygtk_version >= (2,4,0):
+            return FileChooserDialog(
+                title='Save the figure',
+                parent=self.win,
+                filetypes=self.canvas.get_supported_filetypes(),
+                default_filetype=self.canvas.get_default_filetype())
+        else:
+            return FileSelection(title='Save the figure',
+                                 parent=self.win,)
 
     def save_figure(self, button):
-        fname = self.fileselect.get_filename_from_user()
+        fname, format = self.get_filechooser().get_filename_from_user()
         if fname:
-            self.canvas.print_figure(fname)
+            try:
+                self.canvas.print_figure(fname, format=format)
+            except Exception, e:
+                error_msg_gtk(str(e), parent=self)
 
     def configure_subplots(self, button):
         toolfig = Figure(figsize=(6,3))
@@ -637,8 +689,8 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         toolfig.subplots_adjust(top=0.9)
         tool =  SubplotTool(self.canvas.figure, toolfig)
 
-        w = int (toolfig.bbox.width())
-        h = int (toolfig.bbox.height())
+        w = int (toolfig.bbox.width)
+        h = int (toolfig.bbox.height)
 
 
         window = gtk.Window()
@@ -714,8 +766,11 @@ class NavigationToolbar(gtk.Toolbar):
         if gtk.pygtk_version >= (2,4,0):
             self._create_toolitems_2_4()
             self.update = self._update_2_4
-            self.fileselect = FileChooserDialog(title='Save the figure',
-                                                parent=self.win,)
+            self.fileselect = FileChooserDialog(
+                title='Save the figure',
+                parent=self.win,
+                filetypes=self.canvas.get_supported_filetypes(),
+                default_filetype=self.canvas.get_default_filetype())
         else:
             self._create_toolitems_2_2()
             self.update = self._update_2_2
@@ -724,19 +779,19 @@ class NavigationToolbar(gtk.Toolbar):
         self.show_all()
         self.update()
 
-
     def _create_toolitems_2_4(self):
         # use the GTK+ 2.4 GtkToolbar API
         iconSize = gtk.ICON_SIZE_SMALL_TOOLBAR
         self.tooltips = gtk.Tooltips()
 
-        for text, tooltip_text, image, callback, callback_arg, scroll \
+        for text, tooltip_text, image_num, callback, callback_arg, scroll \
                 in self.toolitems:
             if text is None:
                 self.insert( gtk.SeparatorToolItem(), -1 )
                 continue
-            tbutton = gtk.ToolButton(gtk.image_new_from_stock(image, iconSize),
-                                     text)
+            image = gtk.Image()
+            image.set_from_stock(image_num, iconSize)
+            tbutton = gtk.ToolButton(image, text)
             self.insert(tbutton, -1)
             if callback_arg:
                 tbutton.connect('clicked', getattr(self, callback),
@@ -804,13 +859,14 @@ class NavigationToolbar(gtk.Toolbar):
         # use the GTK+ 2.2 (and lower) GtkToolbar API
         iconSize = gtk.ICON_SIZE_SMALL_TOOLBAR
 
-        for text, tooltip_text, image, callback, callback_arg, scroll \
+        for text, tooltip_text, image_num, callback, callback_arg, scroll \
                 in self.toolitems:
             if text is None:
                 self.append_space()
                 continue
-            item = self.append_item(text, tooltip_text, 'Private',
-                                    gtk.image_new_from_stock(image, iconSize),
+            image = gtk.Image()
+            image.set_from_stock(image_num, iconSize)
+            item = self.append_item(text, tooltip_text, 'Private', image,
                                     getattr(self, callback), callback_arg)
             if scroll:
                 item.connect("scroll_event", getattr(self, callback))
@@ -878,85 +934,53 @@ class NavigationToolbar(gtk.Toolbar):
         self._ind = ind
         self._active = [ self._axes[i] for i in self._ind ]
 
-    def panx(self, button, arg):
-        """arg is either user callback data or a scroll event
-        """
-        try:
-            if arg.direction == gdk.SCROLL_UP: direction=1
-            else: direction=-1
-        except AttributeError:
-            direction = arg
+    def panx(self, button, direction):
+        'panx in direction'
 
         for a in self._active:
-            a.panx(direction)
+            a.xaxis.pan(direction)
         self.canvas.draw()
         return True
 
-    def pany(self, button, arg):
-        try:
-            if arg.direction == gdk.SCROLL_UP: direction=1
-            else: direction=-1
-        except AttributeError:
-            direction = arg
-
+    def pany(self, button, direction):
+        'pany in direction'
         for a in self._active:
-            a.pany(direction)
+            a.yaxis.pan(direction)
         self.canvas.draw()
         return True
 
-    def zoomx(self, button, arg):
-        try:
-            if arg.direction == gdk.SCROLL_UP: direction=1
-            else: direction=-1
-        except AttributeError:
-            direction = arg
-
+    def zoomx(self, button, direction):
+        'zoomx in direction'
         for a in self._active:
-            a.zoomx(direction)
+            a.xaxis.zoom(direction)
         self.canvas.draw()
         return True
 
-    def zoomy(self, button, arg):
-        try:
-            if arg.direction == gdk.SCROLL_UP: direction=1
-            else: direction=-1
-        except AttributeError:
-            direction = arg
-
+    def zoomy(self, button, direction):
+        'zoomy in direction'
         for a in self._active:
-            a.zoomy(direction)
+            a.yaxis.zoom(direction)
         self.canvas.draw()
         return True
 
+    def get_filechooser(self):
+        if gtk.pygtk_version >= (2,4,0):
+            return FileChooserDialog(
+                title='Save the figure',
+                parent=self.win,
+                filetypes=self.canvas.get_supported_filetypes(),
+                default_filetype=self.canvas.get_default_filetype())
+        else:
+            return FileSelection(title='Save the figure',
+                                 parent=self.win)
 
     def save_figure(self, button):
-        fname = self.fileselect.get_filename_from_user()
+        fname, format = self.get_filechooser().get_filename_from_user()
         if fname:
-            self.canvas.print_figure(fname)
-
-
-class FileSelection(gtk.FileSelection):
-    """GTK+ 2.2 and lower file selector which remembers the last
-    file/directory selected
-    """
-    def __init__(self, path=None, title='Select a file', parent=None):
-        super(FileSelection, self).__init__(title)
-
-        if path: self.path = path
-        else:    self.path = os.getcwd() + os.sep
-
-        if parent: self.set_transient_for(parent)
-
-    def get_filename_from_user(self, path=None, title=None):
-        if path:  self.path = path
-        if title: self.set_title(title)
-        self.set_filename(self.path)
-
-        filename = None
-        if self.run() == gtk.RESPONSE_OK:
-            self.path = filename = self.get_filename()
-        self.hide()
-        return filename
+            try:
+                self.canvas.print_figure(fname, format=format)
+            except Exception, e:
+                error_msg_gtk(str(e), parent=self)
 
 
 if gtk.pygtk_version >= (2,4,0):
@@ -971,71 +995,98 @@ if gtk.pygtk_version >= (2,4,0):
                       buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                  gtk.STOCK_SAVE,   gtk.RESPONSE_OK),
                       path    = None,
+                      filetypes = [],
+                      default_filetype = None
                       ):
             super (FileChooserDialog, self).__init__ (title, parent, action,
                                                       buttons)
             self.set_default_response (gtk.RESPONSE_OK)
 
-            if path: self.path = path
-            else:    self.path = os.getcwd() + os.sep
+            if not path: path = os.getcwd() + os.sep
 
             # create an extra widget to list supported image formats
-            self.set_current_folder (self.path)
-            self.set_current_name ('image.' + IMAGE_FORMAT_DEFAULT)
+            self.set_current_folder (path)
+            self.set_current_name ('image.' + default_filetype)
 
             hbox = gtk.HBox (spacing=10)
-            hbox.pack_start (gtk.Label ("Image Format:"), expand=False)
+            hbox.pack_start (gtk.Label ("File Format:"), expand=False)
 
-            self.cbox = gtk.combo_box_new_text()
-            hbox.pack_start (self.cbox)
+            liststore = gtk.ListStore(gobject.TYPE_STRING)
+            cbox = gtk.ComboBox(liststore)
+            cell = gtk.CellRendererText()
+            cbox.pack_start(cell, True)
+            cbox.add_attribute(cell, 'text', 0)
+            hbox.pack_start (cbox)
 
-            for item in IMAGE_FORMAT:
-                self.cbox.append_text (item)
-            self.cbox.set_active (IMAGE_FORMAT.index (IMAGE_FORMAT_DEFAULT))
+            self.filetypes = filetypes
+            self.sorted_filetypes = filetypes.items()
+            self.sorted_filetypes.sort()
+            default = 0
+            for i, (ext, name) in enumerate(self.sorted_filetypes):
+                cbox.append_text ("%s (*.%s)" % (name, ext))
+                if ext == default_filetype:
+                    default = i
+            cbox.set_active(default)
+            self.ext = default_filetype
 
             def cb_cbox_changed (cbox, data=None):
                 """File extension changed"""
                 head, filename = os.path.split(self.get_filename())
                 root, ext = os.path.splitext(filename)
                 ext = ext[1:]
-                new_ext = IMAGE_FORMAT[cbox.get_active()]
+                new_ext = self.sorted_filetypes[cbox.get_active()][0]
+                self.ext = new_ext
 
-                if ext in IMAGE_FORMAT:
-                    filename = filename.replace(ext, new_ext)
+                if ext in self.filetypes:
+                    filename = root + '.' + new_ext
                 elif ext == '':
                     filename = filename.rstrip('.') + '.' + new_ext
 
                 self.set_current_name (filename)
-            self.cbox.connect ("changed", cb_cbox_changed)
+            cbox.connect ("changed", cb_cbox_changed)
 
             hbox.show_all()
             self.set_extra_widget(hbox)
 
-
         def get_filename_from_user (self):
             while True:
                 filename = None
-                if self.run() != gtk.RESPONSE_OK:
+                if self.run() != int(gtk.RESPONSE_OK):
                     break
                 filename = self.get_filename()
-                menu_ext  = IMAGE_FORMAT[self.cbox.get_active()]
-                root, ext = os.path.splitext (filename)
-                ext = ext[1:]
-                if ext == '':
-                    ext = menu_ext
-                    filename += '.' + ext
-
-                if ext in IMAGE_FORMAT:
-                    self.path = filename
-                    break
-                else:
-                    error_msg_gtk ('Image format "%s" is not supported' % ext,
-                                   parent=self)
-                    self.set_current_name (os.path.split(root)[1] + '.' +
-                                           menu_ext)
+                break
 
             self.hide()
-            return filename
+            return filename, self.ext
+else:
+    class FileSelection(gtk.FileSelection):
+        """GTK+ 2.2 and lower file selector which remembers the last
+        file/directory selected
+        """
+        def __init__(self, path=None, title='Select a file', parent=None):
+            super(FileSelection, self).__init__(title)
+
+            if path: self.path = path
+            else:    self.path = os.getcwd() + os.sep
+
+            if parent: self.set_transient_for(parent)
+
+        def get_filename_from_user(self, path=None, title=None):
+            if path:  self.path = path
+            if title: self.set_title(title)
+            self.set_filename(self.path)
+
+            filename = None
+            if self.run() == int(gtk.RESPONSE_OK):
+                self.path = filename = self.get_filename()
+            self.hide()
+
+            ext = None
+            if filename is not None:
+                ext = os.path.splitext(filename)[1]
+                if ext.startswith('.'):
+                    ext = ext[1:]
+            return filename, ext
 
 
 class DialogLineprops:
@@ -1052,46 +1103,16 @@ class DialogLineprops:
         'on_dialog_lineprops_cancelbutton_clicked',
         )
 
-    linestyles = (
-        '-'    ,
-        '--'   ,
-        '-.'   ,
-        ','    ,
-        'steps',
-        'None' ,
-    )
-
+    linestyles = [ls for ls in lines.Line2D.lineStyles if ls.strip()]
     linestyled = dict([ (s,i) for i,s in enumerate(linestyles)])
 
 
-    markers =  (
-        'None',
-        '.'  ,
-        ','  ,
-        'o'  ,
-        'v'  ,
-        '^'  ,
-        '<'  ,
-        '>'  ,
-        '1'  ,
-        '2'  ,
-        '3'  ,
-        '4'  ,
-        's'  ,
-        'p'  ,
-        'h'  ,
-        'H'  ,
-        '+'  ,
-        'x'  ,
-        'D'  ,
-        'd'  ,
-        '|'  ,
-        '_'  ,
-        )
+    markers =  [m for m in lines.Line2D.markers if cbook.is_string_like(m)]
 
     markerd = dict([(s,i) for i,s in enumerate(markers)])
 
     def __init__(self, lines):
+        import gtk.glade
 
         datadir = matplotlib.get_data_path()
         gladefile = os.path.join(datadir, 'lineprops.glade')
@@ -1230,11 +1251,17 @@ class DialogLineprops:
     def on_dialog_lineprops_cancelbutton_clicked(self, button):
         self.dlg.hide()
 
-# set icon used when windows are minimized, it requires
-# gtk.pygtk_version >= (2,2,0) with a GDK pixbuf loader for SVG installed
+# set icon used when windows are minimized
+# Unfortunately, the SVG renderer (rsvg) leaks memory under earlier
+# versions of pygtk, so we have to use a PNG file instead.
 try:
+
+    if gtk.pygtk_version < (2, 8, 0):
+        icon_filename = 'matplotlib.png'
+    else:
+        icon_filename = 'matplotlib.svg'
     gtk.window_set_default_icon_from_file (
-        os.path.join (matplotlib.rcParams['datapath'], 'images', 'matplotlib.svg'))
+        os.path.join (matplotlib.rcParams['datapath'], 'images', icon_filename))
 except:
     verbose.report('Could not load matplotlib icon: %s' % sys.exc_info()[1])
 
