@@ -1,49 +1,62 @@
 """
-A class for converting color arguments to RGB or RGBA
+A module for converting numbers or color arguments to *RGB* or *RGBA*
 
-This class instantiates a single instance colorConverter that is used
-to convert matlab color strings to RGB.  RGB is a tuple of float RGB
-values in the range 0-1.
+*RGB* and *RGBA* are sequences of, respectively, 3 or 4 floats in the
+range 0-1.
+
+This module includes functions and classes for color specification
+conversions, and for mapping numbers to colors in a 1-D array of
+colors called a colormap. Colormapping typically involves two steps:
+a data array is first mapped onto the range 0-1 using an instance
+of :class:`Normalize` or of a subclass; then this number in the 0-1
+range is mapped to a color using an instance of a subclass of
+:class:`Colormap`.  Two are provided here:
+:class:`LinearSegmentedColormap`, which is used to generate all
+the built-in colormap instances, but is also useful for making
+custom colormaps, and :class:`ListedColormap`, which is used for
+generating a custom colormap from a list of color specifications.
+
+The module also provides a single instance, *colorConverter*, of the
+:class:`ColorConverter` class providing methods for converting single
+color specifications or sequences of them to *RGB* or *RGBA*.
 
 Commands which take color arguments can use several formats to specify
 the colors.  For the basic builtin colors, you can use a single letter
 
-      b  : blue
-      g  : green
-      r  : red
-      c  : cyan
-      m  : magenta
-      y  : yellow
-      k  : black
-      w  : white
+    - b  : blue
+    - g  : green
+    - r  : red
+    - c  : cyan
+    - m  : magenta
+    - y  : yellow
+    - k  : black
+    - w  : white
 
 Gray shades can be given as a string encoding a float in the 0-1
-range, e.g.,
+range, e.g.::
 
     color = '0.75'
 
 For a greater range of colors, you have two options.  You can specify
-the color using an html hex string, as in
+the color using an html hex string, as in::
 
       color = '#eeefff'
 
-or you can pass an R,G,B tuple, where each of R,G,B are in the range
-[0,1].
+or you can pass an *R* , *G* , *B* tuple, where each of *R* , *G* , *B*
+are in the range [0,1].
 
 Finally, legal html names for colors, like 'red', 'burlywood' and
 'chartreuse' are supported.
 """
 import re
+import numpy as np
+from numpy import ma
+import matplotlib.cbook as cbook
 
-from numerix import array, arange, take, put, Float, Int, putmask, \
-     zeros, asarray, sort, searchsorted, sometrue, ravel, divide,\
-     ones, typecode, typecodes, alltrue, clip
-from numerix.mlab import amin, amax
-import numerix.ma as ma
-import numerix as nx
-from cbook import enumerate, is_string_like, iterable
-from matplotlib import rcParams
-import warnings
+parts = np.__version__.split('.')
+NP_MAJOR, NP_MINOR = map(int, parts[:2])
+# true if clip supports the out kwarg
+NP_CLIP_OUT = NP_MAJOR>=1 and NP_MINOR>=2
 
 cnames = {
     'aliceblue'            : '#F0F8FF',
@@ -195,6 +208,7 @@ for k, v in cnames.items():
         cnames[k] = v
 
 def is_color_like(c):
+    'Return *True* if *c* can be converted to *RGB*'
     try:
         colorConverter.to_rgb(c)
         return True
@@ -210,7 +224,7 @@ hexColorPattern = re.compile("\A#[a-fA-F0-9]{6}\Z")
 
 def hex2color(s):
     """
-    Take a hex string 's' and return the corresponding rgb 3-tuple
+    Take a hex string *s* and return the corresponding rgb 3-tuple
     Example: #efefef -> (0.93725, 0.93725, 0.93725)
     """
     if not isinstance(s, basestring):
@@ -220,6 +234,15 @@ def hex2color(s):
     return tuple([int(n, 16)/255.0 for n in (s[1:3], s[3:5], s[5:7])])
 
 class ColorConverter:
+    """
+    Provides methods for converting color specifications to *RGB* or *RGBA*
+
+    Caching is used for more efficient conversion upon repeated calls
+    with the same argument.
+
+    Ordinarily only the single instance instantiated in this module,
+    *colorConverter*, is needed.
+    """
     colors = {
         'b' : (0.0, 0.0, 1.0),
         'g' : (0.0, 0.5, 0.0),
@@ -234,15 +257,17 @@ class ColorConverter:
     cache = {}
     def to_rgb(self, arg):
         """
-        Returns an RGB tuple of three floats from 0-1.
+        Returns an *RGB* tuple of three floats from 0-1.
+
+        *arg* can be an *RGB* or *RGBA* sequence or a string in any of
+        several forms:
 
-        arg can be an RGB or RGBA sequence or a string in any of several forms:
             1) a letter from the set 'rgbcmykw'
             2) a hex color string, like '#00FFFF'
             3) a standard name, like 'aqua'
             4) a float, like '0.4', indicating gray on a 0-1 scale
 
-        if arg is RGBA, the A will simply be discarded.
+        if *arg* is *RGBA*, the *A* will simply be discarded.
         """
         try: return self.cache[arg]
         except KeyError: pass
@@ -256,7 +281,7 @@ class ColorConverter:
                                     % (str(arg),))
 
         try:
-            if is_string_like(arg):
+            if cbook.is_string_like(arg):
                 color = self.colors.get(arg, None)
                 if color is None:
                     str1 = cnames.get(arg, arg)
@@ -268,7 +293,7 @@ class ColorConverter:
                             raise ValueError(
                                    'gray (string) must be in range 0-1')
                         color = tuple([fl]*3)
-            elif iterable(arg):
+            elif cbook.iterable(arg):
                 if len(arg) > 4 or len(arg) < 3:
                     raise ValueError(
                            'sequence length is %d; must be 3 or 4'%len(arg))
@@ -290,19 +315,23 @@ class ColorConverter:
 
     def to_rgba(self, arg, alpha=None):
         """
-        Returns an RGBA tuple of four floats from 0-1.
+        Returns an *RGBA* tuple of four floats from 0-1.
 
-        For acceptable values of arg, see to_rgb.
-        If arg is an RGBA sequence and alpha is not None,
-        alpha will replace the original A.
+        For acceptable values of *arg*, see :meth:`to_rgb`.
+        If *arg* is an *RGBA* sequence and *alpha* is not *None*,
+        *alpha* will replace the original *A*.
         """
         try:
-            if not is_string_like(arg) and iterable(arg):
-                if len(arg) == 4 and alpha is None:
+            if not cbook.is_string_like(arg) and cbook.iterable(arg):
+                if len(arg) == 4:
                     if [x for x in arg if (float(x) < 0) or  (x > 1)]:
                         # This will raise TypeError if x is not a number.
                         raise ValueError('number in rbga sequence outside 0-1 range')
-                    return tuple(arg)
+                    if alpha is None:
+                        return tuple(arg)
+                    if alpha < 0.0 or alpha > 1.0:
+                        raise ValueError("alpha must be in range 0-1")
+                    return arg[0], arg[1], arg[2], arg[3] * alpha
                 r,g,b = arg[:3]
                 if [x for x in (r,g,b) if (float(x) < 0) or  (x > 1)]:
                     raise ValueError('number in rbg sequence outside 0-1 range')
@@ -314,31 +343,40 @@ class ColorConverter:
         except (TypeError, ValueError), exc:
             raise ValueError('to_rgba: Invalid rgba arg "%s"\n%s' % (str(arg), exc))
 
-    def to_rgba_list(self, c, alpha=None):
+    def to_rgba_array(self, c, alpha=None):
         """
-        Returns a list of rgba tuples.
+        Returns a numpy array of *RGBA* tuples.
 
         Accepts a single mpl color spec or a sequence of specs.
-        If the sequence is a list, the list items are changed in place.
+
+        Special case to handle "no color": if *c* is "none" (case-insensitive),
+        then an empty array will be returned.  Same for an empty list.
         """
         try:
-            return [self.to_rgba(c, alpha)]
+            if c.lower() == 'none':
+                return np.zeros((0,4), dtype=np.float_)
+        except AttributeError:
+            pass
+        if len(c) == 0:
+            return np.zeros((0,4), dtype=np.float_)
+        try:
+            result = np.array([self.to_rgba(c, alpha)], dtype=np.float_)
         except ValueError:
-            # If c is a list it must be maintained as the same list
-            # with modified items so that items can be appended to
-            # it. This is needed for examples/dynamic_collections.py.
-            if not isinstance(c, list): # specific; don't need duck-typing
-                c = list(c)
+            if isinstance(c, np.ndarray):
+                if c.ndim != 2 and c.dtype.kind not in 'SU':
+                    raise ValueError("Color array must be two-dimensional")
+
+            result = np.zeros((len(c), 4))
             for i, cc in enumerate(c):
-                c[i] = self.to_rgba(cc, alpha)  # change in place
-            return c
+                result[i] = self.to_rgba(cc, alpha)  # change in place
+        return np.asarray(result, np.float_)
 
 colorConverter = ColorConverter()
 
 def makeMappingArray(N, data):
-    """Create an N-element 1-d lookup table
+    """Create an *N* -element 1-d lookup table
 
-    data represented by a list of x,y0,y1 mapping correspondences.
+    *data* represented by a list of x,y0,y1 mapping correspondences.
     Each element in this list represents how a value between 0 and 1
     (inclusive) represented by x is mapped to a corresponding value
     between 0 and 1 (inclusive). The two values of y are to allow
@@ -349,11 +387,11 @@ def makeMappingArray(N, data):
     all values of x must be in increasing order. Values between
     the given mapping points are determined by simple linear interpolation.
 
-    The function returns an array "result" where result[x*(N-1)]
+    The function returns an array "result" where ``result[x*(N-1)]``
     gives the closest value for values of x between 0 and 1.
     """
     try:
-        adata = array(data)
+        adata = np.array(data)
     except:
         raise TypeError("data must be convertable to an array")
     shape = adata.shape
@@ -367,22 +405,21 @@ def makeMappingArray(N, data):
     if x[0] != 0. or x[-1] != 1.0:
         raise ValueError(
            "data mapping points must start with x=0. and end with x=1")
-    if sometrue(sort(x)-x):
+    if np.sometrue(np.sort(x)-x):
         raise ValueError(
            "data mapping points must have x in increasing order")
     # begin generation of lookup table
     x = x * (N-1)
-    lut = zeros((N,), Float)
-    xind = arange(float(N))
-    ind = searchsorted(x, xind)[1:-1]
+    lut = np.zeros((N,), np.float)
+    xind = np.arange(float(N))
+    ind = np.searchsorted(x, xind)[1:-1]
 
-    lut[1:-1] = ( divide(xind[1:-1] - take(x,ind-1),
-                         take(x,ind)-take(x,ind-1) )
-                  *(take(y0,ind)-take(y1,ind-1)) + take(y1,ind-1))
+    lut[1:-1] = ( ((xind[1:-1] - x[ind-1]) / (x[ind] - x[ind-1]))
+                  * (y0[ind] - y1[ind-1]) + y1[ind-1])
     lut[0] = y1[0]
     lut[-1] = y0[-1]
     # ensure that the lut is confined to values between 0 and 1 by clipping it
-    clip(lut, 0.0, 1.0)
+    np.clip(lut, 0.0, 1.0)
     #lut = where(lut > 1., 1., lut)
     #lut = where(lut < 0., 0., lut)
     return lut
@@ -392,14 +429,16 @@ class Colormap:
     """Base class for all scalar to rgb mappings
 
         Important methods:
-            set_bad()
-            set_under()
-            set_over()
+
+            * :meth:`set_bad`
+            * :meth:`set_under`
+            * :meth:`set_over`
     """
     def __init__(self, name, N=256):
-        """Public class attributes:
-            self.N:       number of rgb quantization levels
-            self.name:    name of colormap
+        """
+        Public class attributes:
+            :attr:`N` : number of rgb quantization levels
+            :attr:`name` : name of colormap
 
         """
         self.name = name
@@ -413,15 +452,17 @@ class Colormap:
         self._isinit = False
 
 
-    def __call__(self, X, alpha=1.0):
+    def __call__(self, X, alpha=1.0, bytes=False):
         """
-        X is either a scalar or an array (of any dimension).
+        *X* is either a scalar or an array (of any dimension).
         If scalar, a tuple of rgba values is returned, otherwise
         an array with the new shape = oldshape+(4,). If the X-values
         are integers, then they are used as indices into the array.
         If they are floating point, then they must be in the
         interval (0.0, 1.0).
         Alpha must be a scalar.
+        If bytes is False, the rgba values will be floats on a
+        0-1 scale; if True, they will be uint8, 0-255.
         """
 
         if not self._isinit: self._init()
@@ -429,24 +470,39 @@ class Colormap:
         alpha = max(alpha, 0.0)
         self._lut[:-3, -1] = alpha
         mask_bad = None
-        if not iterable(X):
+        if not cbook.iterable(X):
             vtype = 'scalar'
-            xa = array([X])
+            xa = np.array([X])
         else:
             vtype = 'array'
             xma = ma.asarray(X)
             xa = xma.filled(0)
             mask_bad = ma.getmask(xma)
-        if typecode(xa) in typecodes['Float']:
-            putmask(xa, xa==1.0, 0.9999999) #Treat 1.0 as slightly less than 1.
-            xa = (xa * self.N).astype(Int)
+        if xa.dtype.char in np.typecodes['Float']:
+            np.putmask(xa, xa==1.0, 0.9999999) #Treat 1.0 as slightly less than 1.
+            # The following clip is fast, and prevents possible
+            # conversion of large positive values to negative integers.
+
+            if NP_CLIP_OUT:
+                np.clip(xa * self.N, -1, self.N, out=xa)
+            else:
+                xa = np.clip(xa * self.N, -1, self.N)
+            xa = xa.astype(int)
         # Set the over-range indices before the under-range;
         # otherwise the under-range values get converted to over-range.
-        putmask(xa, xa>self.N-1, self._i_over)
-        putmask(xa, xa<0, self._i_under)
+        np.putmask(xa, xa>self.N-1, self._i_over)
+        np.putmask(xa, xa<0, self._i_under)
         if mask_bad is not None and mask_bad.shape == xa.shape:
-            putmask(xa, mask_bad, self._i_bad)
-        rgba = take(self._lut, xa)
+            np.putmask(xa, mask_bad, self._i_bad)
+        if bytes:
+            lut = (self._lut * 255).astype(np.uint8)
+        else:
+            lut = self._lut
+        rgba = np.empty(shape=xa.shape+(4,), dtype=lut.dtype)
+        lut.take(xa, axis=0, mode='clip', out=rgba)
+                    #  twice as fast as lut[xa];
+                    #  using the clip or wrap mode and providing an
+                    #  output array speeds it up a little more.
         if vtype == 'scalar':
             rgba = tuple(rgba[0,:])
         return rgba
@@ -487,25 +543,58 @@ class Colormap:
         raise NotImplementedError("Abstract class only")
 
     def is_gray(self):
-        return (alltrue(self._lut[:,0] == self._lut[:,1])
-                    and alltrue(self._lut[:,0] == self._lut[:,2]))
+        if not self._isinit: self._init()
+        return (np.alltrue(self._lut[:,0] == self._lut[:,1])
+                    and np.alltrue(self._lut[:,0] == self._lut[:,2]))
 
 
 class LinearSegmentedColormap(Colormap):
     """Colormap objects based on lookup tables using linear segments.
 
-    The lookup transfer function is a simple linear function between
-    defined intensities. There is no limit to the number of segments
-    that may be defined. Though as the segment intervals start containing
-    fewer and fewer array locations, there will be inevitable quantization
-    errors
+    The lookup table is generated using linear interpolation for each
+    primary color, with the 0-1 domain divided into any number of
+    segments.
     """
     def __init__(self, name, segmentdata, N=256):
         """Create color map from linear mapping segments
 
         segmentdata argument is a dictionary with a red, green and blue
-        entries. Each entry should be a list of x, y0, y1 tuples.
-        See makeMappingArray for details
+        entries. Each entry should be a list of *x*, *y0*, *y1* tuples,
+        forming rows in a table.
+
+        Example: suppose you want red to increase from 0 to 1 over
+        the bottom half, green to do the same over the middle half,
+        and blue over the top half.  Then you would use::
+
+            cdict = {'red':   [(0.0,  0.0, 0.0),
+                               (0.5,  1.0, 1.0),
+                               (1.0,  1.0, 1.0)],
+
+                     'green': [(0.0,  0.0, 0.0),
+                               (0.25, 0.0, 0.0),
+                               (0.75, 1.0, 1.0),
+                               (1.0,  1.0, 1.0)],
+
+                     'blue':  [(0.0,  0.0, 0.0),
+                               (0.5,  0.0, 0.0),
+                               (1.0,  1.0, 1.0)]}
+
+        Each row in the table for a given color is a sequence of
+        *x*, *y0*, *y1* tuples.  In each sequence, *x* must increase
+        monotonically from 0 to 1.  For any input value *z* falling
+        between *x[i]* and *x[i+1]*, the output value of a given color
+        will be linearly interpolated between *y1[i]* and *y0[i+1]*::
+
+            row i:   x  y0  y1
+                           /
+                          /
+            row i+1: x  y0  y1
+
+        Hence y0 in the first row and y1 in the last row are never used.
+
+
+        .. seealso::
+            :func:`makeMappingArray`
         """
         self.monochrome = False  # True only if all colors in map are identical;
                                  # needed for contouring.
@@ -513,7 +602,7 @@ class LinearSegmentedColormap(Colormap):
         self._segmentdata = segmentdata
 
     def _init(self):
-        self._lut = ones((self.N + 3, 4), Float)
+        self._lut = np.ones((self.N + 3, 4), np.float)
         self._lut[:-3, 0] = makeMappingArray(self.N, self._segmentdata['red'])
         self._lut[:-3, 1] = makeMappingArray(self.N, self._segmentdata['green'])
         self._lut[:-3, 2] = makeMappingArray(self.N, self._segmentdata['blue'])
@@ -521,15 +610,34 @@ class LinearSegmentedColormap(Colormap):
         self._set_extremes()
 
 
-class ListedColormap(LinearSegmentedColormap):
+class ListedColormap(Colormap):
     """Colormap object generated from a list of colors.
 
-    Color boundaries are evenly spaced.  This is intended for simulating
-    indexed color selection, but may be useful for generating
-    special colormaps also.
+    This may be most useful when indexing directly into a colormap,
+    but it can also be used to generate special colormaps for ordinary
+    mapping.
     """
     def __init__(self, colors, name = 'from_list', N = None):
         """
+        Make a colormap from a list of colors.
+
+        *colors*
+            a list of matplotlib color specifications,
+            or an equivalent Nx3 floating point array (*N* rgb values)
+        *name*
+            a string to identify the colormap
+        *N*
+            the number of entries in the map.  The default is *None*,
+            in which case there is one colormap entry for each
+            element in the list of colors.  If::
+
+                N < len(colors)
+
+            the list will be truncated at *N*.  If::
+
+                N > len(colors)
+
+            the list will be extended by repetition.
         """
         self.colors = colors
         self.monochrome = False  # True only if all colors in map are identical;
@@ -537,15 +645,16 @@ class ListedColormap(LinearSegmentedColormap):
         if N is None:
             N = len(self.colors)
         else:
-            if is_string_like(self.colors):
+            if cbook.is_string_like(self.colors):
                 self.colors = [self.colors] * N
                 self.monochrome = True
-            elif iterable(self.colors):
+            elif cbook.iterable(self.colors):
+                self.colors = list(self.colors) # in case it was a tuple
                 if len(self.colors) == 1:
                     self.monochrome = True
                 if len(self.colors) < N:
                     self.colors = list(self.colors) * N
-                    del(self.colors[N:])
+                del(self.colors[N:])
             else:
                 try: gray = float(self.colors)
                 except TypeError: pass
@@ -555,9 +664,9 @@ class ListedColormap(LinearSegmentedColormap):
 
 
     def _init(self):
-        rgb = array([colorConverter.to_rgb(c)
-                    for c in self.colors], Float)
-        self._lut = zeros((self.N + 3, 4), Float)
+        rgb = np.array([colorConverter.to_rgb(c)
+                    for c in self.colors], np.float)
+        self._lut = np.zeros((self.N + 3, 4), np.float)
         self._lut[:-3, :-1] = rgb
         self._lut[:-3, -1] = 1
         self._isinit = True
@@ -568,15 +677,21 @@ class Normalize:
     """
     Normalize a given value to the 0-1 range
     """
-    def __init__(self, vmin=None, vmax=None, clip = True):
+    def __init__(self, vmin=None, vmax=None, clip=False):
         """
-        If vmin or vmax is not given, they are taken from the input's
-        minimum and maximum value respectively.  If clip is True and
+        If *vmin* or *vmax* is not given, they are taken from the input's
+        minimum and maximum value respectively.  If *clip* is *True* and
         the given value falls outside the range, the returned value
-        will be 0 or 1, whichever is closer. Returns 0 if vmin==vmax.
+        will be 0 or 1, whichever is closer. Returns 0 if::
+
+            vmin==vmax
+
         Works with scalars or arrays, including masked arrays.  If
-        clip is True, masked values are set to 1; otherwise they
-        remain masked.
+        *clip* is *True*, masked values are set to 1; otherwise they
+        remain masked.  Clipping silently defeats the purpose of setting
+        the over, under, and masked colors in the colormap, so it is
+        likely to lead to surprises; therefore the default is
+        *clip* = *False*.
         """
         self.vmin = vmin
         self.vmax = vmax
@@ -586,12 +701,12 @@ class Normalize:
         if clip is None:
             clip = self.clip
 
-        if iterable(value):
+        if cbook.iterable(value):
             vtype = 'array'
-            val = ma.asarray(value).astype(Float)
+            val = ma.asarray(value).astype(np.float)
         else:
             vtype = 'scalar'
-            val = ma.array([value]).astype(Float)
+            val = ma.array([value]).astype(np.float)
 
         self.autoscale_None(val)
         vmin, vmax = self.vmin, self.vmax
@@ -602,7 +717,7 @@ class Normalize:
         else:
             if clip:
                 mask = ma.getmask(val)
-                val = ma.array(nx.clip(val.filled(vmax), vmin, vmax),
+                val = ma.array(np.clip(val.filled(vmax), vmin, vmax),
                                 mask=mask)
             result = (val-vmin) * (1.0/(vmax-vmin))
         if vtype == 'scalar':
@@ -614,7 +729,7 @@ class Normalize:
             raise ValueError("Not invertible until scaled")
         vmin, vmax = self.vmin, self.vmax
 
-        if iterable(value):
+        if cbook.iterable(value):
             val = ma.asarray(value)
             return vmin + val * (vmax - vmin)
         else:
@@ -623,7 +738,7 @@ class Normalize:
 
     def autoscale(self, A):
         '''
-        Set vmin, vmax to min, max of A.
+        Set *vmin*, *vmax* to min, max of *A*.
         '''
         self.vmin = ma.minimum(A)
         self.vmax = ma.maximum(A)
@@ -645,12 +760,12 @@ class LogNorm(Normalize):
         if clip is None:
             clip = self.clip
 
-        if iterable(value):
+        if cbook.iterable(value):
             vtype = 'array'
-            val = ma.asarray(value).astype(Float)
+            val = ma.asarray(value).astype(np.float)
         else:
             vtype = 'scalar'
-            val = ma.array([value]).astype(Float)
+            val = ma.array([value]).astype(np.float)
 
         self.autoscale_None(val)
         vmin, vmax = self.vmin, self.vmax
@@ -663,9 +778,9 @@ class LogNorm(Normalize):
         else:
             if clip:
                 mask = ma.getmask(val)
-                val = ma.array(nx.clip(val.filled(vmax), vmin, vmax),
+                val = ma.array(np.clip(val.filled(vmax), vmin, vmax),
                                 mask=mask)
-            result = (ma.log(val)-nx.log(vmin))/(nx.log(vmax)-nx.log(vmin))
+            result = (ma.log(val)-np.log(vmin))/(np.log(vmax)-np.log(vmin))
         if vtype == 'scalar':
             result = result[0]
         return result
@@ -675,17 +790,84 @@ class LogNorm(Normalize):
             raise ValueError("Not invertible until scaled")
         vmin, vmax = self.vmin, self.vmax
 
-        if iterable(value):
+        if cbook.iterable(value):
             val = ma.asarray(value)
             return vmin * ma.power((vmax/vmin), val)
         else:
             return vmin * pow((vmax/vmin), value)
 
+class BoundaryNorm(Normalize):
+    '''
+    Generate a colormap index based on discrete intervals.
+
+    Unlike :class:`Normalize` or :class:`LogNorm`,
+    :class:`BoundaryNorm` maps values to integers instead of to the
+    interval 0-1.
+
+    Mapping to the 0-1 interval could have been done via
+    piece-wise linear interpolation, but using integers seems
+    simpler, and reduces the number of conversions back and forth
+    between integer and floating point.
+    '''
+    def __init__(self, boundaries, ncolors, clip=False):
+        '''
+        *boundaries*
+            a monotonically increasing sequence
+        *ncolors*
+            number of colors in the colormap to be used
+
+        If::
+
+            b[i] <= v < b[i+1]
+
+        then v is mapped to color j;
+        as i varies from 0 to len(boundaries)-2,
+        j goes from 0 to ncolors-1.
+
+        Out-of-range values are mapped to -1 if low and ncolors
+        if high; these are converted to valid indices by
+        :meth:`Colormap.__call__` .
+        '''
+        self.clip = clip
+        self.vmin = boundaries[0]
+        self.vmax = boundaries[-1]
+        self.boundaries = np.asarray(boundaries)
+        self.N = len(self.boundaries)
+        self.Ncmap = ncolors
+        if self.N-1 == self.Ncmap:
+            self._interp = False
+        else:
+            self._interp = True
+
+    def __call__(self, x, clip=None):
+        if clip is None:
+            clip = self.clip
+        x = ma.asarray(x)
+        mask = ma.getmaskarray(x)
+        xx = x.filled(self.vmax+1)
+        if clip:
+            np.clip(xx, self.vmin, self.vmax)
+        iret = np.zeros(x.shape, dtype=np.int16)
+        for i, b in enumerate(self.boundaries):
+            iret[xx>=b] = i
+        if self._interp:
+            iret = (iret * (float(self.Ncmap-1)/(self.N-2))).astype(np.int16)
+        iret[xx<self.vmin] = -1
+        iret[xx>=self.vmax] = self.Ncmap
+        ret = ma.array(iret, mask=mask)
+        if ret.shape == () and not mask:
+            ret = int(ret)  # assume python scalar
+        return ret
+
+    def inverse(self, value):
+        return ValueError("BoundaryNorm is not invertible")
+
 
 class NoNorm(Normalize):
     '''
     Dummy replacement for Normalize, for the case where we
-    want to use indices directly in a ScalarMappable.
+    want to use indices directly in a
+    :class:`~matplotlib.cm.ScalarMappable` .
     '''
     def __call__(self, value, clip=None):
         return value
@@ -696,4 +878,3 @@ class NoNorm(Normalize):
 # compatibility with earlier class names that violated convention:
 normalize = Normalize
 no_norm = NoNorm
-
