@@ -9,29 +9,24 @@ def fn_name(): return sys._getframe(1).f_code.co_name
 import gobject
 import gtk; gdk = gtk.gdk
 import pango
-pygtk_version_required = (2,0,0)
+pygtk_version_required = (2,2,0)
 if gtk.pygtk_version < pygtk_version_required:
-    raise SystemExit ("PyGTK %d.%d.%d is installed\n"
+    raise ImportError ("PyGTK %d.%d.%d is installed\n"
                       "PyGTK %d.%d.%d or later is required"
                       % (gtk.pygtk_version + pygtk_version_required))
 del pygtk_version_required
 
+import numpy as npy
+
 import matplotlib
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
      FigureManagerBase, FigureCanvasBase
-from matplotlib.cbook import is_string_like, enumerate
+from matplotlib.cbook import is_string_like
 from matplotlib.figure import Figure
-from matplotlib.font_manager import fontManager
-from matplotlib.mathtext import math_parse_s_ft2font
-import matplotlib.numerix as numerix
-from matplotlib.numerix import asarray, fromstring, UInt8, zeros, \
-     where, transpose, nonzero, indices, ones, nx
-
-if numerix.which[0] == "numarray":
-    from matplotlib.backends._na_backend_gdk import pixbuf_get_pixels_array
-else:
-    from matplotlib.backends._nc_backend_gdk import pixbuf_get_pixels_array
+from matplotlib.mathtext import MathTextParser
+from matplotlib.transforms import Affine2D
+from matplotlib.backends._backend_gdk import pixbuf_get_pixels_array
 
 
 backend_version = "%d.%d.%d" % gtk.pygtk_version
@@ -76,7 +71,8 @@ class RendererGDK(RendererBase):
         self.gtkDA = gtkDA
         self.dpi   = dpi
         self._cmap = gtkDA.get_colormap()
-        
+        self.mathtext_parser = MathTextParser("Agg")
+
     def set_pixmap (self, pixmap):
         self.gdkDrawable = pixmap
 
@@ -85,30 +81,33 @@ class RendererGDK(RendererBase):
         """
         self.width, self.height = width, height
 
-    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):
-        x, y = int(x-0.5*width), self.height-int(y+0.5*height)
-        w, h = int(width)+1, int(height)+1
-        a1, a2 = int(angle1*64), int(angle2*64)
-        
-        if rgbFace:
-            saveColor = gc.gdkGC.foreground
-            gc.gdkGC.foreground = gc.rgb_to_gdk_color(rgbFace)
-            self.gdkDrawable.draw_arc(gc.gdkGC, True, x, y, w, h, a1, a2)
-            gc.gdkGC.foreground = saveColor
-        self.gdkDrawable.draw_arc(gc.gdkGC, False, x, y, w, h, a1, a2)
+    def draw_path(self, gc, path, transform, rgbFace=None):
+        transform = transform + Affine2D(). \
+            scale(1.0, -1.0).translate(0, self.height)
+        polygons = path.to_polygons(transform, self.width, self.height)
+        for polygon in polygons:
+            # draw_polygon won't take an arbitrary sequence -- it must be a list
+            # of tuples
+            polygon = [(int(round(x)), int(round(y))) for x, y in polygon]
+            if rgbFace is not None:
+                saveColor = gc.gdkGC.foreground
+                gc.gdkGC.foreground = gc.rgb_to_gdk_color(rgbFace)
+                self.gdkDrawable.draw_polygon(gc.gdkGC, True, polygon)
+                gc.gdkGC.foreground = saveColor
+            if gc.gdkGC.line_width > 0:
+                self.gdkDrawable.draw_lines(gc.gdkGC, polygon)
 
-
-    def draw_image(self, x, y, im, origin, bbox):
+    def draw_image(self, x, y, im, bbox, clippath=None, clippath_trans=None):
         if bbox != None:
-            l,b,w,h = bbox.get_bounds()
+            l,b,w,h = bbox.bounds
             #rectangle = (int(l), self.height-int(b+h),
             #             int(w), int(h))
             # set clip rect?
 
-        flipud = origin=='lower'
-        rows, cols, image_str = im.as_str(flipud)
+        im.flipud_out()
+        rows, cols, image_str = im.as_rgba_str()
 
-        image_array = fromstring(image_str, UInt8)
+        image_array = npy.fromstring(image_str, npy.uint8)
         image_array.shape = rows, cols, 4
 
         pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
@@ -120,8 +119,8 @@ class RendererGDK(RendererBase):
 
         gc = self.new_gc()
 
-        if flipud:
-            y = self.height-y-rows
+
+        y = self.height-y-rows
 
         try: # new in 2.2
             # can use None instead of gc.gdkGC, if don't need clipping
@@ -134,43 +133,8 @@ class RendererGDK(RendererBase):
                                   int(x), int(y), cols, rows,
                                   gdk.RGB_DITHER_NONE, 0, 0)
 
-            
-    def draw_line(self, gc, x1, y1, x2, y2):
-        self.gdkDrawable.draw_line(gc.gdkGC, int(x1), self.height-int(y1),
-                                   int(x2), self.height-int(y2))
-
-
-    def draw_lines(self, gc, x, y, transform=None):
-        x = x.astype(nx.Int16)
-        y = self.height - y.astype(nx.Int16)  
-        self.gdkDrawable.draw_lines(gc.gdkGC, zip(x,y))
-        
-
-    def draw_point(self, gc, x, y):
-        self.gdkDrawable.draw_point(gc.gdkGC, int(x), self.height-int(y))
-
-
-    def draw_polygon(self, gc, rgbFace, points):
-        points = [(int(x), self.height-int(y)) for x,y in points]
-        if rgbFace:
-            saveColor = gc.gdkGC.foreground
-            gc.gdkGC.foreground = gc.rgb_to_gdk_color(rgbFace)
-            self.gdkDrawable.draw_polygon(gc.gdkGC, True, points)
-            gc.gdkGC.foreground = saveColor
-        self.gdkDrawable.draw_polygon(gc.gdkGC, False, points)
-
-
-    def draw_rectangle(self, gc, rgbFace, x, y, width, height):
-        x, y = int(x), self.height-int(y+height)
-        #x, y = int(x), self.height-int(math.ceil(y+height))
-        w, h = int(math.ceil(width)), int(math.ceil(height))
-
-        if rgbFace:
-            saveColor = gc.gdkGC.foreground
-            gc.gdkGC.foreground = gc.rgb_to_gdk_color(rgbFace)
-            self.gdkDrawable.draw_rectangle(gc.gdkGC, True, x, y, w, h)
-            gc.gdkGC.foreground = saveColor
-        self.gdkDrawable.draw_rectangle(gc.gdkGC, False, x, y, w, h)
+        # unflip
+        im.flipud_out()
 
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath):
@@ -193,31 +157,28 @@ class RendererGDK(RendererBase):
             l, b, w, h = inkRect
             self.gdkDrawable.draw_layout(gc.gdkGC, x, y-h-b, layout)
 
-        
+
     def _draw_mathtext(self, gc, x, y, s, prop, angle):
-        size = prop.get_size_in_points()
-        width, height, fonts = math_parse_s_ft2font(
-            s, self.dpi.get(), size)
+        ox, oy, width, height, descent, font_image, used_characters = \
+            self.mathtext_parser.parse(s, self.dpi, prop)
 
         if angle==90:
             width, height = height, width
             x -= width
         y -= height
-        
-        imw, imh, image_str = fonts[0].image_as_str()
-        N = imw*imh
+
+        imw = font_image.get_width()
+        imh = font_image.get_height()
+        N = imw * imh
 
         # a numpixels by num fonts array
-        Xall = zeros((N,len(fonts)), typecode=UInt8)
+        Xall = npy.zeros((N,1), npy.uint8)
 
-        for i, font in enumerate(fonts):
-            if angle == 90:
-                font.horiz_image_to_vert_image() # <-- Rotate
-            imw, imh, image_str = font.image_as_str()
-            Xall[:,i] = fromstring(image_str, UInt8)  
+        image_str = font_image.as_str()
+        Xall[:,0] = npy.fromstring(image_str, npy.uint8)
 
         # get the max alpha at each pixel
-        Xs = numerix.mlab.max(Xall,1)
+        Xs = npy.amax(Xall,axis=1)
 
         # convert it to it's proper shape
         Xs.shape = imh, imw
@@ -243,8 +204,8 @@ class RendererGDK(RendererBase):
             pixbuf.render_to_drawable(self.gdkDrawable, gc.gdkGC, 0, 0,
                                   int(x), int(y), imw, imh,
                                   gdk.RGB_DITHER_NONE, 0, 0)
-            
-        
+
+
     def _draw_rotated_text(self, gc, x, y, s, prop, angle):
         """
         Draw the text rotated 90 degrees, other angles are not supported
@@ -308,13 +269,13 @@ class RendererGDK(RendererBase):
         # problem? - cache gets bigger and bigger, is never cleared out
         # two (not one) layouts are created for every text item s (then they
         # are cached) - why?
-        
-        key = self.dpi.get(), s, hash(prop)
+
+        key = self.dpi, s, hash(prop)
         value = self.layoutd.get(key)
         if value != None:
             return value
 
-        size = prop.get_size_in_points() * self.dpi.get() / 96.0
+        size = prop.get_size_in_points() * self.dpi / 96.0
         size = round(size)
 
         font_str = '%s, %s %i' % (prop.get_name(), prop.get_style(), size,)
@@ -324,9 +285,9 @@ class RendererGDK(RendererBase):
         font.set_weight(self.fontweights[prop.get_weight()])
 
         layout = self.gtkDA.create_pango_layout(s)
-        layout.set_font_description(font)    
+        layout.set_font_description(font)
         inkRect, logicalRect = layout.get_pixel_extents()
-        
+
         self.layoutd[key] = layout, inkRect, logicalRect
         return layout, inkRect, logicalRect
 
@@ -337,22 +298,22 @@ class RendererGDK(RendererBase):
     def get_canvas_width_height(self):
         return self.width, self.height
 
-    def get_text_width_height(self, s, prop, ismath):
+    def get_text_width_height_descent(self, s, prop, ismath):
         if ismath:
-            width, height, fonts = math_parse_s_ft2font(
-                s, self.dpi.get(), prop.get_size_in_points())
-            return width, height
+            ox, oy, width, height, descent, font_image, used_characters = \
+                self.mathtext_parser.parse(s, self.dpi, prop)
+            return width, height, descent
 
         layout, inkRect, logicalRect = self._get_pango_layout(s, prop)
         l, b, w, h = inkRect
-        return w, h+1
+        return w, h+1, h + 1
 
     def new_gc(self):
         return GraphicsContextGDK(renderer=self)
 
 
     def points_to_pixels(self, points):
-        return points/72.0 * self.dpi.get()
+        return points/72.0 * self.dpi
 
 
 class GraphicsContextGDK(GraphicsContextBase):
@@ -371,7 +332,7 @@ class GraphicsContextGDK(GraphicsContextBase):
         'round'      : gdk.CAP_ROUND,
         }
 
-              
+
     def __init__(self, renderer):
         GraphicsContextBase.__init__(self)
         self.renderer = renderer
@@ -385,9 +346,9 @@ class GraphicsContextGDK(GraphicsContextBase):
         return an allocated gtk.gdk.Color
         """
         try:
-            return self._cached[rgb] 
+            return self._cached[tuple(rgb)]
         except KeyError:
-            color = self._cached[rgb] = \
+            color = self._cached[tuple(rgb)] = \
                     self._cmap.alloc_color(
                         int(rgb[0]*65535),int(rgb[1]*65535),int(rgb[2]*65535))
             return color
@@ -403,13 +364,14 @@ class GraphicsContextGDK(GraphicsContextBase):
 
     def set_clip_rectangle(self, rectangle):
         GraphicsContextBase.set_clip_rectangle(self, rectangle)
-        l,b,w,h = rectangle
+        if rectangle is None:
+            return
+        l,b,w,h = rectangle.bounds
         rectangle = (int(l), self.renderer.height-int(b+h)+1,
                      int(w), int(h))
         #rectangle = (int(l), self.renderer.height-int(b+h),
         #             int(w+1), int(h+2))
-        self.gdkGC.set_clip_rectangle(rectangle)        
-
+        self.gdkGC.set_clip_rectangle(rectangle)
 
     def set_dashes(self, dash_offset, dash_list):
         GraphicsContextBase.set_dashes(self, dash_offset, dash_list)
@@ -417,7 +379,7 @@ class GraphicsContextGDK(GraphicsContextBase):
         if dash_list == None:
             self.gdkGC.line_style = gdk.LINE_SOLID
         else:
-            pixels = self.renderer.points_to_pixels(asarray(dash_list))
+            pixels = self.renderer.points_to_pixels(npy.asarray(dash_list))
             dl = [max(1, int(round(val))) for val in pixels]
             self.gdkGC.set_dashes(dash_offset, dl)
             self.gdkGC.line_style = gdk.LINE_ON_OFF_DASH
@@ -431,7 +393,7 @@ class GraphicsContextGDK(GraphicsContextBase):
     def set_graylevel(self, frac):
         GraphicsContextBase.set_graylevel(self, frac)
         self.gdkGC.foreground = self.rgb_to_gdk_color(self.get_rgb())
-        
+
 
     def set_joinstyle(self, js):
         GraphicsContextBase.set_joinstyle(self, js)
@@ -440,15 +402,19 @@ class GraphicsContextGDK(GraphicsContextBase):
 
     def set_linewidth(self, w):
         GraphicsContextBase.set_linewidth(self, w)
-        pixels = self.renderer.points_to_pixels(w)
-        self.gdkGC.line_width = max(1, int(round(pixels)))
+        if w == 0:
+            self.gdkGC.line_width = 0
+        else:
+            pixels = self.renderer.points_to_pixels(w)
+            self.gdkGC.line_width = max(1, int(round(pixels)))
+
 
-                                               
 def new_figure_manager(num, *args, **kwargs):
     """
     Create a new figure manager instance
     """
-    thisFig = Figure(*args, **kwargs)
+    FigureClass = kwargs.pop('FigureClass', Figure)
+    thisFig = FigureClass(*args, **kwargs)
     canvas  = FigureCanvasGDK(thisFig)
     manager = FigureManagerBase(canvas, num)
     # equals:
@@ -460,7 +426,7 @@ def new_figure_manager(num, *args, **kwargs):
 class FigureCanvasGDK (FigureCanvasBase):
     def __init__(self, figure):
         FigureCanvasBase.__init__(self, figure)
-        
+
         self._renderer_init()
 
     def _renderer_init(self):
@@ -471,53 +437,30 @@ class FigureCanvasGDK (FigureCanvasBase):
         self._renderer.set_width_height (width, height)
         self.figure.draw (self._renderer)
 
-    def print_figure(self, filename, dpi=150, facecolor='w', edgecolor='w',
-                     orientation='portrait'):
-        root, ext = os.path.splitext(filename)       
-        ext = ext[1:]
-        if ext == '':
-            ext      = IMAGE_FORMAT_DEFAULT
-            filename = filename + '.' + ext        
+    filetypes = FigureCanvasBase.filetypes.copy()
+    filetypes['jpg'] = 'JPEG'
+    filetypes['jpeg'] = 'JPEG'
 
-        self.figure.dpi.set(dpi)        
-        self.figure.set_facecolor(facecolor)
-        self.figure.set_edgecolor(edgecolor)
+    def print_jpeg(self, filename, *args, **kwargs):
+        return self._print_image(filename, 'jpeg')
+    print_jpg = print_jpeg
 
-        ext = ext.lower()
-        if ext in ('jpg', 'png'):          # native printing
-            width, height = self.get_width_height()
-            pixmap = gtk.gdk.Pixmap (None, width, height, depth=24)
-            self._render_figure(pixmap, width, height)
+    def print_png(self, filename, *args, **kwargs):
+        return self._print_image(filename, 'png')
 
-            # jpg colors don't match the display very well, png colors match
-            # better
-            pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8,
-                                    width, height)
-            pixbuf.get_from_drawable(pixmap, pixmap.get_colormap(),
-                                     0, 0, 0, 0, width, height)
-        
-            # pixbuf.save() recognises 'jpeg' not 'jpg'
-            if ext == 'jpg': ext = 'jpeg' 
+    def _print_image(self, filename, format, *args, **kwargs):
+        width, height = self.get_width_height()
+        pixmap = gtk.gdk.Pixmap (None, width, height, depth=24)
+        self._render_figure(pixmap, width, height)
 
-            pixbuf.save(filename, ext)
+        # jpg colors don't match the display very well, png colors match
+        # better
+        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8,
+                                width, height)
+        pixbuf.get_from_drawable(pixmap, pixmap.get_colormap(),
+                                 0, 0, 0, 0, width, height)
 
-        elif ext in ('eps', 'ps', 'svg',):
-            if ext == 'svg':
-                from backend_svg import FigureCanvasSVG as FigureCanvas
-            else:
-                from backend_ps  import FigureCanvasPS  as FigureCanvas
+        pixbuf.save(filename, format)
 
-            
-            fc = self.switch_backends(FigureCanvas)
-            fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
-        elif ext in ('bmp', 'raw', 'rgb',):
-            
-            from backend_agg import FigureCanvasAgg  as FigureCanvas
-            fc = self.switch_backends(FigureCanvas)
-            fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
-
-        else:
-            raise ValueError('Format "%s" is not supported.\nSupported formats are %s.' %
-                             (ext, ', '.join(IMAGE_FORMAT)))
-
-        self.figure.set_canvas(self)
+    def get_default_filetype(self):
+        return 'png'
