@@ -1,115 +1,188 @@
 '''
 Colorbar toolkit with two classes and a function:
 
-    ColorbarBase is the base class with full colorbar drawing functionality.
+    :class:`ColorbarBase`
+        the base class with full colorbar drawing functionality.
         It can be used as-is to make a colorbar for a given colormap;
         a mappable object (e.g., image) is not needed.
 
-    Colorbar is the derived class for use with images or contour plots.
+    :class:`Colorbar`
+        the derived class for use with images or contour plots.
 
-    make_axes is a function for resizing an axes and adding a second axes
+    :func:`make_axes`
+        a function for resizing an axes and adding a second axes
         suitable for a colorbar
 
-The Figure.colorbar() method uses make_axes and Colorbar; the pylab.colorbar()
-function is a thin wrapper over Figure.colorbar().
+The :meth:`~matplotlib.figure.Figure.colorbar` method uses :func:`make_axes`
+and :class:`Colorbar`; the :func:`~matplotlib.pyplot.colorbar` function
+is a thin wrapper over :meth:`~matplotlib.figure.Figure.colorbar`.
 
 '''
 
-import matplotlib.numerix as nx
-from matplotlib.mlab import meshgrid, linspace
-from matplotlib.numerix.mlab import amin, amax
-from matplotlib import colors, cm, ticker
-from matplotlib.cbook import iterable, is_string_like
-from matplotlib.transforms import Interval, Value, PBox
-from matplotlib.lines import Line2D
-from matplotlib.patches import Polygon
-from matplotlib import rcParams
-from matplotlib.collections import LineCollection
-from matplotlib.contour import ContourSet
-from matplotlib.axes import Axes
+import numpy as np
+import matplotlib as mpl
+import matplotlib.colors as colors
+import matplotlib.cm as cm
+import matplotlib.ticker as ticker
+import matplotlib.cbook as cbook
+import matplotlib.lines as lines
+import matplotlib.patches as patches
+import matplotlib.collections as collections
+import matplotlib.contour as contour
 
 make_axes_kw_doc = '''
-        fraction    = 0.15; fraction of original axes to use for colorbar
-        pad         = 0.05 if vertical, 0.15 if horizontal; fraction
-                              of original axes between colorbar and
-                              new image axes
-        shrink      = 1.0; fraction by which to shrink the colorbar
-        aspect      = 20; ratio of long to short dimensions
+
+    ==========   ====================================================
+    Property     Description
+    ==========   ====================================================
+    *fraction*   0.15; fraction of original axes to use for colorbar
+    *pad*        0.05 if vertical, 0.15 if horizontal; fraction
+                 of original axes between colorbar and new image axes
+    *shrink*     1.0; fraction by which to shrink the colorbar
+    *aspect*     20; ratio of long to short dimensions
+    ==========   ====================================================
 
 '''
 
 colormap_kw_doc = '''
-        extend='neither', 'both', 'min', 'max'
-                If not 'neither', make pointed end(s) for out-of-range
-                values.  These are set for a given colormap using the
-                colormap set_under and set_over methods.
-        spacing='uniform', 'proportional'
-                Uniform spacing gives each discrete color the same space;
-                proportional makes the space proportional to the data interval.
-        ticks=None, list of ticks, Locator object
-                If None, ticks are determined automatically from the input.
-        format=None, format string, Formatter object
-                If none, the ScalarFormatter is used.
-                If a format string is given, e.g. '%.3f', that is used.
-                An alternative Formatter object may be given instead.
-        drawedges=False, True
-                If true, draw lines at color boundaries.
 
-        The following will probably be useful only in the context of
-        indexed colors (that is, when the mappable has norm=NoNorm()),
-        or other unusual circumstances.
+    ===========   ====================================================
+    Property      Description
+    ===========   ====================================================
+    *extend*      [ 'neither' | 'both' | 'min' | 'max' ]
+                  If not 'neither', make pointed end(s) for out-of-
+                  range values.  These are set for a given colormap
+                  using the colormap set_under and set_over methods.
+    *spacing*     [ 'uniform' | 'proportional' ]
+                  Uniform spacing gives each discrete color the same
+                  space; proportional makes the space proportional to
+                  the data interval.
+    *ticks*       [ None | list of ticks | Locator object ]
+                  If None, ticks are determined automatically from the
+                  input.
+    *format*      [ None | format string | Formatter object ]
+                  If None, the
+                  :class:`~matplotlib.ticker.ScalarFormatter` is used.
+                  If a format string is given, e.g. '%.3f', that is
+                  used. An alternative
+                  :class:`~matplotlib.ticker.Formatter` object may be
+                  given instead.
+    *drawedges*   [ False | True ] If true, draw lines at color
+                  boundaries.
+    ===========   ====================================================
 
-        boundaries=None or a sequence
-        values=None or a sequence which must be of length 1 less than the
-                sequence of boundaries.
-                For each region delimited by adjacent entries in
-                boundaries, the color mapped to the corresponding
-                value in values will be used.
+    The following will probably be useful only in the context of
+    indexed colors (that is, when the mappable has norm=NoNorm()),
+    or other unusual circumstances.
+
+    ============   ===================================================
+    Property       Description
+    ============   ===================================================
+    *boundaries*   None or a sequence
+    *values*       None or a sequence which must be of length 1 less
+                   than the sequence of *boundaries*. For each region
+                   delimited by adjacent entries in *boundaries*, the
+                   color mapped to the corresponding value in values
+                   will be used.
+    ============   ===================================================
 
 '''
 
 colorbar_doc = '''
+
 Add a colorbar to a plot.
 
-Function signatures:
+Function signatures for the :mod:`~matplotlib.pyplot` interface; all
+but the first are also method signatures for the
+:meth:`~matplotlib.figure.Figure.colorbar` method::
 
-    colorbar(**kwargs)
+  colorbar(**kwargs)
+  colorbar(mappable, **kwargs)
+  colorbar(mappable, cax=cax, **kwargs)
+  colorbar(mappable, ax=ax, **kwargs)
 
-    colorbar(mappable, **kwargs)
+arguments:
 
-    colorbar(mappable, cax, **kwargs)
+  *mappable*
+    the :class:`~matplotlib.image.Image`,
+    :class:`~matplotlib.contour.ContourSet`, etc. to
+    which the colorbar applies; this argument is mandatory for the
+    :meth:`~matplotlib.figure.Figure.colorbar` method but optional for the
+    :func:`~matplotlib.pyplot.colorbar` function, which sets the
+    default to the current image.
 
-The optional arguments mappable and cax may be included in the kwargs;
-they are image, ContourSet, etc. to which the colorbar applies, and
-the axes object in which the colorbar will be drawn.  Defaults are
-the current image and a new axes object created next to that image
-after resizing the image.
+keyword arguments:
 
-kwargs are in two groups:
-    axes properties:
+  *cax*
+    None | axes object into which the colorbar will be drawn
+  *ax*
+    None | parent axes object from which space for a new
+    colorbar axes will be stolen
+
+
+Additional keyword arguments are of two kinds:
+
+  axes properties:
 %s
-    colorbar properties:
+  colorbar properties:
 %s
 
-If mappable is a ContourSet, its extend kwarg is included automatically.
+If *mappable* is a :class:`~matplotlib.contours.ContourSet`, its *extend*
+kwarg is included automatically.
 
-Note that the shrink kwarg provides a simple way to keep
-a vertical colorbar, for example, from being taller than
-the axes of the mappable to which the colorbar is attached;
-but it is a manual method requiring some trial and error.
-If the colorbar is too tall (or a horizontal colorbar is
-too wide) use a smaller value of shrink.
+Note that the *shrink* kwarg provides a simple way to keep a vertical
+colorbar, for example, from being taller than the axes of the mappable
+to which the colorbar is attached; but it is a manual method requiring
+some trial and error. If the colorbar is too tall (or a horizontal
+colorbar is too wide) use a smaller value of *shrink*.
+
+For more precise control, you can manually specify the positions of
+the axes objects in which the mappable and the colorbar are drawn.  In
+this case, do not use any of the axes properties kwargs.
+
+returns:
+    :class:`~matplotlib.colorbar.Colorbar` instance; see also its base class,
+    :class:`~matplotlib.colorbar.ColorbarBase`.  Call the
+    :meth:`~matplotlib.colorbar.ColorbarBase.set_label` method
+    to label the colorbar.
 
-For more precise control, you can manually specify the
-positions of the axes objects in which the mappable and
-the colorbar are drawn.  In this case, do not use any of the
-axes properties kwargs.
 ''' % (make_axes_kw_doc, colormap_kw_doc)
 
 
 
 class ColorbarBase(cm.ScalarMappable):
     '''
+    Draw a colorbar in an existing axes.
+
+    This is a base class for the :class:`Colorbar` class, which is the
+    basis for the :func:`~matplotlib.pyplot.colorbar` method and pylab
+    function.
+
+    It is also useful by itself for showing a colormap.  If the *cmap*
+    kwarg is given but *boundaries* and *values* are left as None,
+    then the colormap will be displayed on a 0-1 scale. To show the
+    under- and over-value colors, specify the *norm* as::
+
+        colors.Normalize(clip=False)
+
+    To show the colors versus index instead of on the 0-1 scale,
+    use::
+
+        norm=colors.NoNorm.
+
+    Useful attributes:
+
+        :attr:`ax`
+            the Axes instance in which the colorbar is drawn
+
+        :attr:`lines`
+            a LineCollection if lines were drawn, otherwise None
+
+        :attr:`dividers`
+            a LineCollection if *drawedges* is True, otherwise None
+
+    Useful public methods are :meth:`set_label` and :meth:`add_lines`.
+
     '''
     _slice_dict = {'neither': slice(0,1000000),
                    'both': slice(1,-1),
@@ -144,7 +217,9 @@ class ColorbarBase(cm.ScalarMappable):
         self.filled = filled
         self.solids = None
         self.lines = None
-        if iterable(ticks):
+        self.dividers = None
+        self.set_label('')
+        if cbook.iterable(ticks):
             self.locator = ticker.FixedLocator(ticks, nbins=len(ticks))
         else:
             self.locator = ticks    # Handle default in _ticker()
@@ -153,7 +228,7 @@ class ColorbarBase(cm.ScalarMappable):
                 self.formatter = ticker.LogFormatter()
             else:
                 self.formatter = ticker.ScalarFormatter()
-        elif is_string_like(format):
+        elif cbook.is_string_like(format):
             self.formatter = ticker.FormatStrFormatter(format)
         else:
             self.formatter = format  # Assume it is a Formatter
@@ -168,10 +243,11 @@ class ColorbarBase(cm.ScalarMappable):
         self._process_values()
         self._find_range()
         X, Y = self._mesh()
-        C = self._values[:,nx.NewAxis]
+        C = self._values[:,np.newaxis]
         self._config_axes(X, Y)
         if self.filled:
             self._add_solids(X, Y, C)
+        self._set_label()
 
     def _config_axes(self, X, Y):
         '''
@@ -180,15 +256,17 @@ class ColorbarBase(cm.ScalarMappable):
         ax = self.ax
         ax.set_frame_on(False)
         ax.set_navigate(False)
-        x, y = self._outline(X, Y)
-        ax.set_xlim(amin(x), amax(x))
-        ax.set_ylim(amin(y), amax(y))
-        ax.update_datalim_numerix(x, y)
-        self.outline = Line2D(x, y, color=rcParams['axes.edgecolor'],
-                                    linewidth=rcParams['axes.linewidth'])
+        xy = self._outline(X, Y)
+        ax.update_datalim(xy)
+        ax.set_xlim(*ax.dataLim.intervalx)
+        ax.set_ylim(*ax.dataLim.intervaly)
+        self.outline = lines.Line2D(xy[:, 0], xy[:, 1], color=mpl.rcParams['axes.edgecolor'],
+                                    linewidth=mpl.rcParams['axes.linewidth'])
         ax.add_artist(self.outline)
-        c = rcParams['axes.facecolor']
-        self.patch = Polygon(zip(x,y), edgecolor=c,
+        self.outline.set_clip_box(None)
+        self.outline.set_clip_path(None)
+        c = mpl.rcParams['axes.facecolor']
+        self.patch = patches.Polygon(xy, edgecolor=c,
                  facecolor=c,
                  linewidth=0.01,
                  zorder=-1)
@@ -209,30 +287,41 @@ class ColorbarBase(cm.ScalarMappable):
             ax.set_xticklabels(ticklabels)
             ax.xaxis.get_major_formatter().set_offset_string(offset_string)
 
-    def set_label(self, label, **kw):
+    def _set_label(self):
         if self.orientation == 'vertical':
-            self.ax.set_ylabel(label, **kw)
+            self.ax.set_ylabel(self._label, **self._labelkw)
         else:
-            self.ax.set_xlabel(label, **kw)
+            self.ax.set_xlabel(self._label, **self._labelkw)
+
+    def set_label(self, label, **kw):
+        '''
+        Label the long axis of the colorbar
+        '''
+        self._label = label
+        self._labelkw = kw
+        self._set_label()
+
 
     def _outline(self, X, Y):
         '''
-        Return x, y arrays of colorbar bounding polygon,
+        Return *x*, *y* arrays of colorbar bounding polygon,
         taking orientation into account.
         '''
-        N = nx.shape(X)[0]
+        N = X.shape[0]
         ii = [0, 1, N-2, N-1, 2*N-1, 2*N-2, N+1, N, 0]
-        x = nx.take(nx.ravel(nx.transpose(X)), ii)
-        y = nx.take(nx.ravel(nx.transpose(Y)), ii)
+        x = np.take(np.ravel(np.transpose(X)), ii)
+        y = np.take(np.ravel(np.transpose(Y)), ii)
+        x = x.reshape((len(x), 1))
+        y = y.reshape((len(y), 1))
         if self.orientation == 'horizontal':
-            return y,x
-        return x,y
+            return np.hstack((y, x))
+        return np.hstack((x, y))
 
     def _edges(self, X, Y):
         '''
         Return the separator line segments; helper for _add_solids.
         '''
-        N = nx.shape(X)[0]
+        N = X.shape[0]
         # Using the non-array form of these line segments is much
         # simpler than making them into arrays.
         if self.orientation == 'vertical':
@@ -242,24 +331,30 @@ class ColorbarBase(cm.ScalarMappable):
 
     def _add_solids(self, X, Y, C):
         '''
-        Draw the colors using pcolor; optionally add separators.
+        Draw the colors using :meth:`~matplotlib.axes.Axes.pcolor`;
+        optionally add separators.
         '''
-        ## Change to pcolormesh if/when it is fixed to handle alpha
-        ## correctly.
+        ## Change to pcolorfast after fixing bugs in some backends...
         if self.orientation == 'vertical':
             args = (X, Y, C)
         else:
-            args = (nx.transpose(Y), nx.transpose(X), nx.transpose(C))
+            args = (np.transpose(Y), np.transpose(X), np.transpose(C))
         kw = {'cmap':self.cmap, 'norm':self.norm,
                     'shading':'flat', 'alpha':self.alpha}
+        # Save, set, and restore hold state to keep pcolor from
+        # clearing the axes. Ordinarily this will not be needed,
+        # since the axes object should already have hold set.
+        _hold = self.ax.ishold()
+        self.ax.hold(True)
         col = self.ax.pcolor(*args, **kw)
+        self.ax.hold(_hold)
         #self.add_observer(col) # We should observe, not be observed...
         self.solids = col
         if self.drawedges:
-            self.dividers = LineCollection(self._edges(X,Y),
-                                           colors=(rcParams['axes.edgecolor'],),
-                                           linewidths=(0.5*rcParams['axes.linewidth'],)
-                                           )
+            self.dividers = collections.LineCollection(self._edges(X,Y),
+                              colors=(mpl.rcParams['axes.edgecolor'],),
+                              linewidths=(0.5*mpl.rcParams['axes.linewidth'],)
+                              )
             self.ax.add_collection(self.dividers)
 
     def add_lines(self, levels, colors, linewidths):
@@ -270,13 +365,13 @@ class ColorbarBase(cm.ScalarMappable):
         dummy, y = self._locate(levels)
         if len(y) <> N:
             raise ValueError("levels are outside colorbar range")
-        x = nx.array([0.0, 1.0])
-        X, Y = meshgrid(x,y)
+        x = np.array([0.0, 1.0])
+        X, Y = np.meshgrid(x,y)
         if self.orientation == 'vertical':
             xy = [zip(X[i], Y[i]) for i in range(N)]
         else:
             xy = [zip(Y[i], X[i]) for i in range(N)]
-        col = LineCollection(xy, linewidths=linewidths)
+        col = collections.LineCollection(xy, linewidths=linewidths)
         self.lines = col
         col.set_color(colors)
         self.ax.add_collection(col)
@@ -295,6 +390,9 @@ class ColorbarBase(cm.ScalarMappable):
                     nv = len(self._values)
                     base = 1 + int(nv/10)
                     locator = ticker.IndexLocator(base=base, offset=0)
+                elif isinstance(self.norm, colors.BoundaryNorm):
+                    b = self.norm.boundaries
+                    locator = ticker.FixedLocator(b, nbins=10)
                 elif isinstance(self.norm, colors.LogNorm):
                     locator = ticker.LogLocator()
                 else:
@@ -303,66 +401,103 @@ class ColorbarBase(cm.ScalarMappable):
                 b = self._boundaries[self._inside]
                 locator = ticker.FixedLocator(b, nbins=10)
         if isinstance(self.norm, colors.NoNorm):
-            intv = Interval(Value(self._values[0]), Value(self._values[-1]))
+            intv = self._values[0], self._values[-1]
         else:
-            intv = Interval(Value(self.vmin), Value(self.vmax))
-        locator.set_view_interval(intv)
-        locator.set_data_interval(intv)
-        formatter.set_view_interval(intv)
-        formatter.set_data_interval(intv)
-        b = nx.array(locator())
+            intv = self.vmin, self.vmax
+        locator.create_dummy_axis()
+        formatter.create_dummy_axis()
+        locator.set_view_interval(*intv)
+        locator.set_data_interval(*intv)
+        formatter.set_view_interval(*intv)
+        formatter.set_data_interval(*intv)
+        b = np.array(locator())
         b, ticks = self._locate(b)
         formatter.set_locs(b)
-        ticklabels = [formatter(t) for t in b]
+        ticklabels = [formatter(t, i) for i, t in enumerate(b)]
         offset_string = formatter.get_offset()
         return ticks, ticklabels, offset_string
 
     def _process_values(self, b=None):
         '''
-        Set the _boundaries and _values attributes based on
-        the input boundaries and values.  Input boundaries can
-        be self.boundaries or the argument b.
+        Set the :attr:`_boundaries` and :attr:`_values` attributes
+        based on the input boundaries and values.  Input boundaries
+        can be *self.boundaries* or the argument *b*.
         '''
         if b is None:
             b = self.boundaries
         if b is not None:
-            self._boundaries = nx.array(b)
+            self._boundaries = np.asarray(b, dtype=float)
             if self.values is None:
                 self._values = 0.5*(self._boundaries[:-1]
                                         + self._boundaries[1:])
                 if isinstance(self.norm, colors.NoNorm):
-                    self._values = (self._values + 0.00001).astype(nx.Int16)
+                    self._values = (self._values + 0.00001).astype(np.int16)
                 return
-            self._values = nx.array(self.values)
+            self._values = np.array(self.values)
             return
         if self.values is not None:
-            self._values = nx.array(self.values)
+            self._values = np.array(self.values)
             if self.boundaries is None:
-                b = nx.zeros(len(self.values)+1, 'd')
+                b = np.zeros(len(self.values)+1, 'd')
                 b[1:-1] = 0.5*(self._values[:-1] - self._values[1:])
                 b[0] = 2.0*b[1] - b[2]
                 b[-1] = 2.0*b[-2] - b[-3]
                 self._boundaries = b
                 return
-            self._boundaries = nx.array(self.boundaries)
+            self._boundaries = np.array(self.boundaries)
             return
+        # Neither boundaries nor values are specified;
+        # make reasonable ones based on cmap and norm.
         if isinstance(self.norm, colors.NoNorm):
-            b = nx.arange(self.norm.vmin, self.norm.vmax + 2) - 0.5
+            b = self._uniform_y(self.cmap.N+1) * self.cmap.N - 0.5
+            v = np.zeros((len(b)-1,), dtype=np.int16)
+            v[self._inside] = np.arange(self.cmap.N, dtype=np.int16)
+            if self.extend in ('both', 'min'):
+                v[0] = -1
+            if self.extend in ('both', 'max'):
+                v[-1] = self.cmap.N
+            self._boundaries = b
+            self._values = v
+            return
+        elif isinstance(self.norm, colors.BoundaryNorm):
+            b = list(self.norm.boundaries)
+            if self.extend in ('both', 'min'):
+                b = [b[0]-1] + b
+            if self.extend in ('both', 'max'):
+                b = b + [b[-1] + 1]
+            b = np.array(b)
+            v = np.zeros((len(b)-1,), dtype=float)
+            bi = self.norm.boundaries
+            v[self._inside] = 0.5*(bi[:-1] + bi[1:])
+            if self.extend in ('both', 'min'):
+                v[0] = b[0] - 1
+            if self.extend in ('both', 'max'):
+                v[-1] = b[-1] + 1
+            self._boundaries = b
+            self._values = v
+            return
         else:
+            if not self.norm.scaled():
+                self.norm.vmin = 0
+                self.norm.vmax = 1
             b = self.norm.inverse(self._uniform_y(self.cmap.N+1))
+            if self.extend in ('both', 'min'):
+                b[0] = b[0] - 1
+            if self.extend in ('both', 'max'):
+                b[-1] = b[-1] + 1
         self._process_values(b)
 
     def _find_range(self):
         '''
-        Set vmin and vmax attributes to the first and last
-        boundary excluding extended end boundaries.
+        Set :attr:`vmin` and :attr:`vmax` attributes to the first and
+        last boundary excluding extended end boundaries.
         '''
         b = self._boundaries[self._inside]
         self.vmin = b[0]
         self.vmax = b[-1]
 
     def _central_N(self):
-        '''number of boundaries *before* extension of ends'''
+        '''number of boundaries **before** extension of ends'''
         nb = len(self._boundaries)
         if self.extend == 'both':
             nb -= 2
@@ -384,23 +519,23 @@ class ColorbarBase(cm.ScalarMappable):
 
     def _uniform_y(self, N):
         '''
-        Return colorbar data coordinates for N uniformly
+        Return colorbar data coordinates for *N* uniformly
         spaced boundaries, plus ends if required.
         '''
         if self.extend == 'neither':
-            y = linspace(0, 1, N)
+            y = np.linspace(0, 1, N)
         else:
             if self.extend == 'both':
-                y = nx.zeros(N + 2, 'd')
+                y = np.zeros(N + 2, 'd')
                 y[0] = -0.05
                 y[-1] = 1.05
             elif self.extend == 'min':
-                y = nx.zeros(N + 1, 'd')
+                y = np.zeros(N + 1, 'd')
                 y[0] = -0.05
             else:
-                y = nx.zeros(N + 1, 'd')
+                y = np.zeros(N + 1, 'd')
                 y[-1] = 1.05
-            y[self._inside] = linspace(0, 1, N)
+            y[self._inside] = np.linspace(0, 1, N)
         return y
 
     def _proportional_y(self):
@@ -408,7 +543,12 @@ class ColorbarBase(cm.ScalarMappable):
         Return colorbar data coordinates for the boundaries of
         a proportional colorbar.
         '''
-        y = self.norm(self._boundaries.copy())
+        if isinstance(self.norm, colors.BoundaryNorm):
+            b = self._boundaries[self._inside]
+            y = (self._boundaries - self._boundaries[0])
+            y = y / (self._boundaries[-1] - self._boundaries[0])
+        else:
+            y = self.norm(self._boundaries.copy())
         if self.extend in ('both', 'min'):
             y[0] = -0.05
         if self.extend in ('both', 'max'):
@@ -425,13 +565,13 @@ class ColorbarBase(cm.ScalarMappable):
         transposition for a horizontal colorbar are done outside
         this function.
         '''
-        x = nx.array([0.0, 1.0])
+        x = np.array([0.0, 1.0])
         if self.spacing == 'uniform':
             y = self._uniform_y(self._central_N())
         else:
             y = self._proportional_y()
         self._y = y
-        X, Y = meshgrid(x,y)
+        X, Y = np.meshgrid(x,y)
         if self.extend in ('min', 'both'):
             X[0,:] = 0.5
         if self.extend in ('max', 'both'):
@@ -444,7 +584,7 @@ class ColorbarBase(cm.ScalarMappable):
         within range, together with their corresponding colorbar
         data coordinates.
         '''
-        if isinstance(self.norm, colors.NoNorm):
+        if isinstance(self.norm, (colors.NoNorm, colors.BoundaryNorm)):
             b = self._boundaries
             xn = x
             xout = x
@@ -457,19 +597,19 @@ class ColorbarBase(cm.ScalarMappable):
             # floating point errors.
             xn = self.norm(x, clip=False).filled()
             in_cond = (xn > -0.001) & (xn < 1.001)
-            xn = nx.compress(in_cond, xn)
-            xout = nx.compress(in_cond, x)
+            xn = np.compress(in_cond, xn)
+            xout = np.compress(in_cond, x)
         # The rest is linear interpolation with clipping.
         y = self._y
         N = len(b)
-        ii = nx.minimum(nx.searchsorted(b, xn), N-1)
-        i0 = nx.maximum(ii - 1, 0)
-        #db = b[ii] - b[i0]  (does not work with Numeric)
-        db = nx.take(b, ii) - nx.take(b, i0)
-        db = nx.where(i0==ii, 1.0, db)
+        ii = np.minimum(np.searchsorted(b, xn), N-1)
+        i0 = np.maximum(ii - 1, 0)
+        #db = b[ii] - b[i0]
+        db = np.take(b, ii) - np.take(b, i0)
+        db = np.where(i0==ii, 1.0, db)
         #dy = y[ii] - y[i0]
-        dy = nx.take(y, ii) - nx.take(y, i0)
-        z = nx.take(y, i0) + (xn-nx.take(b,i0))*dy/db
+        dy = np.take(y, ii) - np.take(y, i0)
+        z = np.take(y, i0) + (xn-np.take(b,i0))*dy/db
         return xout, z
 
     def set_alpha(self, alpha):
@@ -486,7 +626,7 @@ class Colorbar(ColorbarBase):
         kw['cmap'] = mappable.cmap
         kw['norm'] = mappable.norm
         kw['alpha'] = mappable.get_alpha()
-        if isinstance(mappable, ContourSet):
+        if isinstance(mappable, contour.ContourSet):
             CS = mappable
             kw['boundaries'] = CS._levels
             kw['values'] = CS.cvalues
@@ -503,9 +643,10 @@ class Colorbar(ColorbarBase):
 
     def add_lines(self, CS):
         '''
-        Add the lines from a non-filled ContourSet to the colorbar.
+        Add the lines from a non-filled
+        :class:`~matplotlib.contour.ContourSet` to the colorbar.
         '''
-        if not isinstance(CS, ContourSet) or CS.filled:
+        if not isinstance(CS, contour.ContourSet) or CS.filled:
             raise ValueError('add_lines is only for a ContourSet of lines')
         tcolors = [c[0] for c in CS.tcolors]
         tlinewidths = [t[0] for t in CS.tlinewidths]
@@ -519,18 +660,22 @@ class Colorbar(ColorbarBase):
         #print 'tlinewidths:', tlinewidths
         ColorbarBase.add_lines(self, CS.levels, tcolors, tlinewidths)
 
-    def notify(self, mappable):
-        '''Manually change any contour line colors.  This is called
+    def update_bruteforce(self, mappable):
+        '''
+        Manually change any contour line colors.  This is called
         when the image or contour plot to which this colorbar belongs
         is changed.
         '''
-        cm.ScalarMappable.notify(self, mappable)
+        # We are using an ugly brute-force method: clearing and
+        # redrawing the whole thing.  The problem is that if any
+        # properties have been changed by methods other than the
+        # colorbar methods, those changes will be lost.
         self.ax.cla()
         self.draw_all()
         #if self.vmin != self.norm.vmin or self.vmax != self.norm.vmax:
         #    self.ax.cla()
         #    self.draw_all()
-        if isinstance(self.mappable, ContourSet):
+        if isinstance(self.mappable, contour.ContourSet):
             CS = self.mappable
             if not CS.filled:
                 self.add_lines(CS)
@@ -547,19 +692,19 @@ def make_axes(parent, **kw):
     fraction = kw.pop('fraction', 0.15)
     shrink = kw.pop('shrink', 1.0)
     aspect = kw.pop('aspect', 20)
-    #pb = PBox(parent.get_position())
-    pb = PBox(parent.get_position(original=True))
+    #pb = transforms.PBox(parent.get_position())
+    pb = parent.get_position(original=True).frozen()
     if orientation == 'vertical':
         pad = kw.pop('pad', 0.05)
         x1 = 1.0-fraction
         pb1, pbx, pbcb = pb.splitx(x1-pad, x1)
-        pbcb.shrink(1.0, shrink).anchor('C')
+        pbcb = pbcb.shrunk(1.0, shrink).anchored('C', pbcb)
         anchor = (0.0, 0.5)
         panchor = (1.0, 0.5)
     else:
         pad = kw.pop('pad', 0.15)
         pbcb, pbx, pb1 = pb.splity(fraction, fraction+pad)
-        pbcb.shrink(shrink, 1.0).anchor('C')
+        pbcb = pbcb.shrunk(shrink, 1.0).anchored('C', pbcb)
         aspect = 1.0/aspect
         anchor = (0.5, 1.0)
         panchor = (0.5, 0.0)
@@ -571,12 +716,15 @@ def make_axes(parent, **kw):
     return cax, kw
 make_axes.__doc__ ='''
     Resize and reposition a parent axes, and return a child
-    axes suitable for a colorbar.
+    axes suitable for a colorbar::
 
-    cax, kw = make_axes(parent, **kw)
+        cax, kw = make_axes(parent, **kw)
 
     Keyword arguments may include the following (with defaults):
-        orientation = 'vertical'  or 'horizontal'
+
+        *orientation*
+            'vertical'  or 'horizontal'
+
     %s
 
     All but the first of these are stripped from the input kw set.
@@ -585,37 +733,3 @@ make_axes.__doc__ ='''
     '''  % make_axes_kw_doc
 
 
-'''
-The following does not work correctly.  The problem seems to be that
-the transforms work right only when fig.add_axes(rect) is used to
-generate the axes, not when the axes object is generated first and
-then fig.add_axes(ax) is called.  I don't understand this. - EF
-
-class ColorbarAxes(Axes):
-    def __init__(self, parent, **kw):
-        orientation = kw.setdefault('orientation', 'vertical')
-        fraction = kw.pop('fraction', 0.15)
-        shrink = kw.pop('shrink', 1.0)
-        aspect = kw.pop('aspect', 20)
-        self.cbkw = kw
-        pb = PBox(parent.get_position())
-        if orientation == 'vertical':
-            pb1, pbcb = pb.splitx(1.0-fraction)
-            pbcb.shrink(1.0, shrink).anchor('C')
-            anchor = (0.3, 0.5)
-            panchor = (0.8, 0.5)
-        else:
-            pbcb, pb1 = pb.splity(fraction)
-            pbcb.shrink(shrink, 1.0).anchor('C')
-            aspect = 1.0/aspect
-            anchor = (0.5, 0.2)
-            panchor = (0.5, 0.8)
-        parent.set_position(pb1)
-        parent.set_anchor(panchor)
-        fig = parent.get_figure()
-        Axes.__init__(self, fig, pbcb)
-        fig.add_axes(self)
-        self.set_aspect(aspect, anchor=anchor, adjustable='box')
-
-'''
-
