@@ -20,18 +20,14 @@ License   : matplotlib license
 
 """
 from __future__ import division
-import sys, warnings
-from matplotlib import verbose
-from numerix import ones, Float, add, asarray
+import warnings
 
 import artist
 from artist import Artist
 from patches import Rectangle
-from cbook import enumerate, is_string_like, flatten
+from cbook import is_string_like
 from text import Text
-from transforms import Bbox, inverse_transform_bbox, bbox_all, unit_bbox, \
-     get_bbox_transform
-
+from transforms import Bbox
 
 
 
@@ -47,6 +43,7 @@ class Cell(Rectangle):
                  fill=True,
                  text='',
                  loc=None,
+                 fontproperties=None
                  ):
 
         # Call base
@@ -58,7 +55,8 @@ class Cell(Rectangle):
         # Create text object
         if loc is None: loc = 'right'
         self._loc = loc
-        self._text = Text(x=xy[0], y=xy[1], text=text)
+        self._text = Text(x=xy[0], y=xy[1], text=text,
+                          fontproperties=fontproperties)
         self._text.set_clip_on(False)
 
 
@@ -75,7 +73,6 @@ class Cell(Rectangle):
         return self._text
 
     def set_fontsize(self, size):
-
         self._text.set_fontsize(size)
 
     def get_fontsize(self):
@@ -108,7 +105,7 @@ class Cell(Rectangle):
         Currently support 'left', 'center' and 'right'
         """
         bbox = self.get_window_extent(renderer)
-        l, b, w, h = bbox.get_bounds()
+        l, b, w, h = bbox.bounds
 
         # draw in center vertically
         self._text.set_verticalalignment('center')
@@ -130,8 +127,8 @@ class Cell(Rectangle):
     def get_text_bounds(self, renderer):
         """ Get text bounds in axes co-ordinates. """
         bbox = self._text.get_window_extent(renderer)
-        bboxa = inverse_transform_bbox(self.get_transform(), bbox)
-        return bboxa.get_bounds()
+        bboxa = bbox.inverse_transformed(self.get_data_transform())
+        return bboxa.bounds
 
     def get_required_width(self, renderer):
         """ Get width required for this cell. """
@@ -184,7 +181,7 @@ class Table(Artist):
 
         Artist.__init__(self)
 
-        if is_string_like(loc) and not self.codes.has_key(loc):
+        if is_string_like(loc) and loc not in self.codes:
             warnings.warn('Unrecognized location %s. Falling back on bottom; valid locations are\n%s\t' %(loc, '\n\t'.join(self.codes.keys())))
             loc = 'bottom'
         if is_string_like(loc): loc = self.codes.get(loc, 1)
@@ -202,6 +199,8 @@ class Table(Artist):
         self._autoColumns = []
         self._autoFontsize = True
 
+        self._cachedRenderer = None
+
     def add_cell(self, row, col, *args, **kwargs):
         """ Add a cell to the table. """
         xy = (0,0)
@@ -214,9 +213,16 @@ class Table(Artist):
         self._cells[(row, col)] = cell
 
     def _approx_text_height(self):
-        return self.FONTSIZE/72.0*self.figure.dpi.get()/self._axes.bbox.height() * 1.2
+        return self.FONTSIZE/72.0*self.figure.dpi/self._axes.bbox.height * 1.2
 
     def draw(self, renderer):
+        # Need a renderer to do hit tests on mouseevent; assume the last one will do
+        if renderer is None:
+            renderer = self._cachedRenderer
+        if renderer is None:
+            raise RuntimeError('No renderer defined')
+        self._cachedRenderer = renderer
+
         if not self.get_visible(): return
         renderer.open_group('table')
         self._update_positions(renderer)
@@ -237,12 +243,31 @@ class Table(Artist):
                  for pos in self._cells.keys()
                  if pos[0] >= 0 and pos[1] >= 0]
 
-        bbox = bbox_all(boxes)
-        return inverse_transform_bbox(self.get_transform(), bbox)
+        bbox = Bbox.union(boxes)
+        return bbox.inverse_transformed(self.get_transform())
 
-    def get_child_artists(self):
-        'Return the Artists cintained by the table'
+    def contains(self,mouseevent):
+        """Test whether the mouse event occurred in the table.
+
+        Returns T/F, {}
+        """
+        if callable(self._contains): return self._contains(self,mouseevent)
+
+        # TODO: Return index of the cell containing the cursor so that the user
+        # doesn't have to bind to each one individually.
+        if self._cachedRenderer is not None:
+            boxes = [self._cells[pos].get_window_extent(self._cachedRenderer)
+                 for pos in self._cells.keys()
+                 if pos[0] >= 0 and pos[1] >= 0]
+            bbox = bbox_all(boxes)
+            return bbox.contains(mouseevent.x,mouseevent.y),{}
+        else:
+            return False,{}
+
+    def get_children(self):
+        'Return the Artists contained by the table'
         return self._cells.values()
+    get_child_artists = get_children  # backward compatibility
 
     def get_window_extent(self, renderer):
         'Return the bounding box of the table in window coords'
@@ -365,7 +390,7 @@ class Table(Artist):
         self._do_cell_alignment()
 
         bbox = self._get_grid_bbox(renderer)
-        l,b,w,h = bbox.get_bounds()
+        l,b,w,h = bbox.bounds
 
         if self._bbox is not None:
             # Position according to bbox
@@ -502,7 +527,7 @@ def table(ax,
     if rowLabels is not None:
         for row in xrange(rows):
             table.add_cell(row+offset, -1,
-                           width=rowLabelWidth, height=height,
+                           width=rowLabelWidth or 1e-15, height=height,
                            text=rowLabels[row], facecolor=rowColours[row],
                            loc=rowLoc)
         if rowLabelWidth == 0:
