@@ -1,8 +1,9 @@
+#!/usr/bin/env python2.5
+
 from matplotlib.widgets import Cursor
 import pylab
 
 
-        
 fig = pylab.figure(figsize=(8,6))
 ax = fig.add_axes([0.075, 0.25, 0.9, 0.725], axisbg='#FFFFCC')
 #ax = fig.add_subplot(111, axisbg='#FFFFCC')
@@ -13,7 +14,7 @@ ax.plot(x,y,'o')
 ax.set_xlim(-2,2)
 ax.set_ylim(-2,2)
 
-# set useblit = True on gtkagg for enhanced performance    
-cursor = Cursor(ax, useblit=False, color='red', linewidth=2 )
+# set useblit = True on gtkagg for enhanced performance
+cursor = Cursor(ax, useblit=True, color='red', linewidth=2 )
 
 pylab.show()
